import asyncio
import json
import logging
from typing import Any
from typing import TypedDict

import botocore.exceptions
from aiobotocore.session import get_session

DOCUMENTATION = r"""
---
short_description: Receive events via an AWS SQS queue
description:
  - An ansible-rulebook event source plugin for receiving events via an AWS SQS queue.
  - >
    This supports all the authentication methods supported by boto3 library:
    U(https://boto3.amazonaws.com/v1/documentation/api/latest/guide/credentials.html).
options:
  access_key:
    description:
      - Optional AWS access key ID.
    type: str
  secret_key:
    description:
      - Optional AWS secret access key.
    type: str
  session_token:
    description:
      - Optional STS session token for use with temporary credentials.
    type: str
  endpoint_url:
    description:
      - Optional URL to connect to instead of the default AWS endpoints.
    type: str
  queue_owner_aws_account_id:
    description:
      - >
        Optional The Amazon Web Services account ID of the account that
        created the queue. This is only required when you are attempting
        to access a queue owned by another Amazon Web Services account.
    type: str
  region:
    description:
      - Optional AWS region to use.
    type: str
  name:
    description:
      - Name of the queue.
    type: str
    required: true
  delay_seconds:
    description:
      - The SQS long polling duration.
      - Set to V(0) to disable.
    type: int
    default: 2
  max_number_of_messages:
    description:
      - >
        Optional The maximum number of messages to return. Amazon SQS
        never returns more messages than this value (however, fewer
        messages might be returned). Valid values are V(1) to V(10).
    type: int
    default: 1
  feedback:
    type: bool
    default: false
    description:
      - Should the source plugin wait for feedback before
        processing the next event from SQS Queue.
        This flag allows ansible rulebook to pass in an asyncio
        queue which is passed in the O(eda_feedback_queue).
        The source plugin should wait for the response to come
        back on this queue before it picks the next event from
        SQS queue.
    version_added: 11.3.0
  feedback_timeout:
    type: int
    default: 120
    description:
      - Timeout in seconds to wait for feedback from the rule engine
        before raising an exception. Only applies when feedback is enabled.
    version_added: 11.3.0
  eda_feedback_queue:
    description:
      - Provided automatically by ansible-rulebook when the feedback
        parameter is enabled, this parameter utilizes an asyncio.Queue.
        It allows the system to wait for confirmation that an event has
        been safely persisted in the database before removing it from the
        event queue. Users do not need to provide a value for this manually.
    version_added: 11.3.0
"""

EXAMPLES = r"""
- amazon.aws.aws_sqs_queue:
    region: us-east-1
    name: eda
    delay_seconds: 10
"""


DEFAULT_FEEDBACK_TIMEOUT = 120


class DeleteMessageBatchRequestEntryTypeDef(TypedDict):
    """Custom TypedDict for Delete Message Batch Request Entry."""

    Id: str
    ReceiptHandle: str


class MessageDeletionException(Exception):
    pass


class QueueConfig(TypedDict):
    """Configuration extracted from arguments."""

    queue_name: str
    wait_seconds: int
    max_number_of_messages: int
    queue_owner_aws_account_id: str | None
    feedback_enabled: bool
    feedback_timeout: int
    eda_feedback_queue: asyncio.Queue[Any] | None


def _validate_and_extract_config(args: dict[str, Any]) -> QueueConfig:
    """Validate arguments and extract configuration.

    Args:
        args: Dictionary of arguments

    Returns:
        QueueConfig with validated configuration

    Raises:
        ValueError: If required arguments are missing or invalid
    """
    if "name" not in args:
        msg = "Missing queue name"
        raise ValueError(msg)

    feedback_enabled = args.get("feedback", False)
    eda_feedback_queue = args.get("eda_feedback_queue", None)

    if feedback_enabled and eda_feedback_queue is None:
        msg = (
            "feedback: true was set but no feedback queue was provided. "
            "This requires a compatible version of ansible-rulebook that "
            "supports the feedback mechanism."
        )
        raise ValueError(msg)

    return QueueConfig(
        queue_name=str(args.get("name")),
        wait_seconds=int(args.get("delay_seconds", 2)),
        max_number_of_messages=int(args.get("max_number_of_messages", 1)),
        queue_owner_aws_account_id=args.get("queue_owner_aws_account_id"),
        feedback_enabled=feedback_enabled,
        feedback_timeout=int(args.get("feedback_timeout", DEFAULT_FEEDBACK_TIMEOUT)),
        eda_feedback_queue=eda_feedback_queue,
    )


async def _get_queue_url(
    client,
    queue_name: str,
    queue_owner_aws_account_id: str | None,
) -> str:
    """Retrieve the SQS queue URL.

    Args:
        client: AWS SQS client
        queue_name: Name of the queue
        queue_owner_aws_account_id: Optional account ID of queue owner

    Returns:
        Queue URL

    Raises:
        ValueError: If queue does not exist
    """
    queue_args = {"QueueName": queue_name}
    if queue_owner_aws_account_id:
        queue_args["QueueOwnerAWSAccountId"] = queue_owner_aws_account_id

    try:
        response = await client.get_queue_url(**queue_args)
    except botocore.exceptions.ClientError as err:
        if err.response["Error"]["Code"] == "AWS.SimpleQueueService.NonExistentQueue":
            msg = f"Queue {queue_name} does not exist"
            raise ValueError(msg) from None
        raise

    return response["QueueUrl"]


def _parse_message_body(entry: dict[str, Any]) -> Any:
    """Parse the message body from JSON.

    Args:
        entry: SQS message entry

    Returns:
        Parsed message body or raw body if parsing fails
    """
    logger = logging.getLogger()

    try:
        return json.loads(entry["Body"])
    except json.JSONDecodeError:
        logger.error("JSON decode error, storing raw value")
        return entry["Body"]
    except UnicodeError:
        logger.exception("Unicode error while decoding message body")
        return None


async def _handle_feedback_and_delete(
    client,
    queue_url: str,
    delete_entry: DeleteMessageBatchRequestEntryTypeDef,
    config: QueueConfig,
) -> None:
    """Handle feedback mechanism and message deletion.

    Args:
        client: AWS SQS client
        queue_url: Queue URL
        delete_entry: Message deletion entry
        config: Queue configuration

    Raises:
        asyncio.TimeoutError: If feedback timeout is exceeded
        MessageDeletionException: If message deletion fails
    """
    logger = logging.getLogger()

    if config["feedback_enabled"] and config["eda_feedback_queue"]:
        try:
            await asyncio.wait_for(
                config["eda_feedback_queue"].get(),
                timeout=config["feedback_timeout"],
            )
            if not await _delete_messages(client, queue_url, [delete_entry]):
                logger.exception("Failed to delete messages, exiting")
                raise MessageDeletionException("Messages could not be deleted")
        except asyncio.TimeoutError:
            logger.exception("Timed out waiting for feedback")
            raise
    else:
        await _delete_messages(client, queue_url, [delete_entry])


async def _process_message(
    entry: dict[str, Any],
    queue: asyncio.Queue[Any],
    client,
    queue_url: str,
    config: QueueConfig,
) -> None:
    """Process a single SQS message.

    Args:
        entry: SQS message entry
        queue: Queue to put processed events
        client: AWS SQS client
        queue_url: Queue URL
        config: Queue configuration

    Raises:
        ValueError: If message format is invalid
    """
    if not isinstance(entry, dict) or "MessageId" not in entry:  # pragma: no cover
        err_msg = f"Unexpected response {entry}, missing MessageId."
        raise ValueError(err_msg)

    delete_entry = DeleteMessageBatchRequestEntryTypeDef(
        Id=entry["MessageId"],
        ReceiptHandle=entry["ReceiptHandle"],
    )

    meta = {
        "MessageId": entry["MessageId"],
        "event": {"uuid": entry["MessageId"]},
    }

    msg_body = _parse_message_body(entry)

    await queue.put({"body": msg_body, "meta": meta})
    await _handle_feedback_and_delete(client, queue_url, delete_entry, config)
    await asyncio.sleep(0)


async def _delete_messages(client, queue_url, entries) -> bool:
    result = False
    logger = logging.getLogger()
    # Need to remove msg from queue or else it'll reappear
    delete_results = await client.delete_message_batch(
        QueueUrl=queue_url,
        Entries=entries,
    )

    # Check if the deletion was successful
    if "Successful" in delete_results:
        result = True
        for item in delete_results["Successful"]:
            logger.debug(
                "Message with ID %s deleted successfully",
                item["Id"],
            )
    # Log a error if any message failed deletions
    if "Failed" in delete_results:
        result = False
        for item in delete_results["Failed"]:
            logger.error(
                "Error deleting message with ID : %s Error Code: %s Error Message: %s SenderFault: %s",
                item["Id"],
                item["Code"],
                item["Message"],
                str(item["SenderFault"]),
            )

    return result


async def main(
    queue: asyncio.Queue[Any],
    args: dict[str, Any],
) -> None:
    """Receive events via an AWS SQS queue."""
    logger = logging.getLogger()

    # Validate arguments and extract configuration
    config = _validate_and_extract_config(args)

    session = get_session()

    async with session.create_client("sqs", **connection_args(args)) as client:
        # Get the queue URL
        queue_url = await _get_queue_url(
            client,
            config["queue_name"],
            config["queue_owner_aws_account_id"],
        )

        # Main message processing loop
        while True:
            # This loop won't spin really fast as there is
            # essentially a sleep in the receive_message call
            response_msg = await client.receive_message(
                QueueUrl=queue_url,
                WaitTimeSeconds=config["wait_seconds"],
                MaxNumberOfMessages=config["max_number_of_messages"],
            )

            if "Messages" in response_msg:
                for entry in response_msg["Messages"]:
                    await _process_message(entry, queue, client, queue_url, config)
            else:
                logger.debug("No messages in queue")


def connection_args(args: dict[str, Any]) -> dict[str, Any]:
    """Provide connection arguments to AWS SQS queue."""
    selected_args = {}

    # Best Practice: get credentials from ~/.aws/credentials or the environment
    # However the following method are still possible
    if "access_key" in args:
        selected_args["aws_access_key_id"] = args["access_key"]
    if "secret_key" in args:
        selected_args["aws_secret_access_key"] = args["secret_key"]
    if "session_token" in args:
        selected_args["aws_session_token"] = args["session_token"]

    if "endpoint_url" in args:
        selected_args["endpoint_url"] = args["endpoint_url"]
    if "region" in args:
        selected_args["region_name"] = args["region"]

    return selected_args


if __name__ == "__main__":
    # MockQueue if running directly

    class MockQueue(asyncio.Queue[Any]):
        """A fake queue."""

        async def put(self: "MockQueue", event: dict[str, Any]) -> None:
            """Print the event."""
            print(event)  # noqa: T201

    asyncio.run(main(MockQueue(), {"region": "us-east-1", "name": "eda"}))



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_ml_compute
version_added: "3.15.0"
short_description: Create, Update or Delete an Azure Machine Learning Compute
description:
    - Create, Update or Delete an Azure Machine Learning Compute.
options:
    name:
        description:
            - Name of the Azure ML compute.
        type: str
        required: true
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    ml_workspace:
        description:
            - Name of the Azure ML workspace.
        required: true
        type: str
    description:
        description:
            - Description of the compute target.
        type: str
        required: false
    resource_definition:
        description:
            - Compute definition in YAML
        type: str
        required: false
    identity_type:
        description:
            - The type of managed identity.
        type: str
        choices:
            - SystemAssigned
            - UserAssigned
        required: false
    size:
        description:
            - "VM size to use for the compute target. More details can be
              found here: https://aka.ms/azureml-vm-details."
        type: str
        required: false
    ssh_key_value:
        description:
            - SSH public key of the administrator user account.
        type: str
        required: false
    ssh_public_access_enabled:
        description:
            - Indicates whether public SSH port is enabled.
        type: bool
        required: false
    subnet:
        description:
            - Name of the subnet. Can also reference a subnet in an
              existing vnet by ID instead of name. If subnet ID is
              specified then vnet_name will be ignored. Subnet ID can refer
              to a vnet/subnet in another RG by specifying the fully
              qualified subnet ID. Required when vnet name is specified.
        type: str
        required: false
    type:
        description:
            - The type of compute target.
        choices:
            - amlcompute
            - computeinstance
        type: str
        required: false
    user_assigned_identities:
        description:
            - User Assigned Identities.
        type: list
        elements: str
    vnet_name:
        description:
            - Name of the virtual network.
        type: str
        required: false
    admin_username:
        description:
            - Name of the administrator user account that can be used to
              SSH into the node(s).
        type: str
        required: false
    admin_password:
        description:
            - Password for the administrator user account.
        type: str
        required: false
    idle_time_before_scale_down:
        description:
            - Node idle time in seconds before scaling down the cluster.
        type: int
        required: false
    location:
        description:
            - The location to be used for the new compute.
              If not specified, defaults to the location of the workspace.
        type: str
        required: false
    max_instances:
        description:
            - The maximum number of nodes to use on the cluster.
        required: false
        type: int
    min_instances:
        description:
            - The minimum number of nodes to use on the cluster.
        required: false
        type: int
    tier:
        description:
            - VM priority tier.
        choices:
            - dedicated
            - low_priority
        type: str
    user_object_id:
        description:
            - AAD object ID of the assigned user.
        type: str
        required: false
    user_tenant_id:
        description:
            - AAD tenant ID of the assigned user.
        type: str
        required: false
    state:
        description:
            - State of the Compute. Use C(present) to create
              or update and C(absent) to delete. C(start) to
              start, C(stop) to stop and C(restart) to restart.
        default: present
        type: str
        choices:
            - absent
            - present
            - start
            - stop
            - restart

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: Create ML Compute
  azure.azcollection.azure_rm_ml_compute:
    location: eastus
    name: MyCompute
    description: My Compute created by Ansible
    resource_group: myResourceGroup
    ml_workspace: myMLWorkspace
    type: amlcompute
    min_instances: 0
    max_instances: 8
    tags:
      createdByToolkit: Ansible
'''

RETURN = '''
changed:
    description:
        - Whether the resource is changed.
    returned: always
    type: bool
    sample: false
ml_compute:
    description:
        - Compute that was just created or updated.
    returned: always
    type: dict
    sample: {
      "description": "Compute Integration Tests",
      "enable_node_public_ip": true,
      "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-compute/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxx-xxx/computes/amlxxxxxxx-xxxcomp",
      "idle_time_before_scale_down": 120,
      "location": "eastus",
      "max_instances": 200,
      "min_instances": 0,
      "name": "amlxxxxxxx-xxxcomp",
      "network_settings": {},
      "provisioning_state": "Succeeded",
      "size": "STANDARD_DS3_V2",
      "ssh_public_access_enabled": true,
      "tags": {
        "created_by": "Ansible"
      },
      "tier": "dedicated",
      "type": "amlcompute"
    }
'''  # NOQA


try:
    import io
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ml import MLClientCommon
    from azure.core.exceptions import ResourceNotFoundError
    from azure.ai.ml.entities._load_functions import load_compute
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMMLCompute(MLClientCommon):

    UPDATE_OVERRIDES = ['min_instances',
                        'max_instances',
                        'idle_time_before_scale_down',
                        'identity_type',
                        'user_assigned_identities',
                        'tags'
                        ]

    PARAM_OVERRIDES = UPDATE_OVERRIDES + ['name',
                                          'type',
                                          'vnet_name',
                                          'subnet',
                                          'admin_username',
                                          'admin_password',
                                          'ssh_key_value',
                                          'size',
                                          'user_tenant_id',
                                          'user_object_id',
                                          'description',
                                          'enable_node_public_ip',
                                          'tier',
                                          'ssh_public_access_enabled',
                                          'location',
                                          ]

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(
                type='str',
                required=True,
            ),
            description=dict(
                type='str',
                required=False,
            ),
            resource_definition=dict(
                type='str',
                required=False,
            ),
            location=dict(
                type='str',
                required=False,
            ),
            resource_group=dict(
                type='str',
                required=True,
            ),
            ml_workspace=dict(
                type='str',
                required=True,
            ),
            identity_type=dict(
                type='str',
                required=False,
                choices=['SystemAssigned', 'UserAssigned'],
            ),
            size=dict(
                type='str',
                required=False,
            ),
            ssh_key_value=dict(
                type='str',
                required=False,
                no_log=True,
            ),
            ssh_public_access_enabled=dict(
                type='bool',
                required=False,
            ),
            subnet=dict(
                type='str',
                required=False,
            ),
            type=dict(
                choices=['amlcompute', 'computeinstance'],
                type='str',
                required=False,
            ),
            user_assigned_identities=dict(
                type='list',
                elements='str',
                required=False,
            ),
            vnet_name=dict(
                type='str',
                required=False,
            ),
            admin_username=dict(
                type='str',
                required=False,
            ),
            admin_password=dict(
                type='str',
                required=False,
                no_log=True,
            ),
            idle_time_before_scale_down=dict(
                type='int',
                required=False,
            ),
            max_instances=dict(
                type='int',
                required=False,
            ),
            min_instances=dict(
                type='int',
                required=False,
            ),
            tier=dict(
                choices=['dedicated', 'low_priority'],
                type='str',
                required=False,
            ),
            user_object_id=dict(
                type='str',
                required=False,
            ),
            user_tenant_id=dict(
                type='str',
                required=False,
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent', 'start', 'stop', 'restart'],
            ),
        )

        self._client = None
        self.ml_compute = None
        self.ml_registry = None
        self.state = None

        self.results = dict(
            changed=False,
            compare=[],
            ml_compute={}
        )

        super(AzureRMMLCompute, self).__init__(self.module_arg_spec,
                                               supports_check_mode=True,
                                               )

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        if self.resource_definition:
            resource_definition = io.StringIO(self.resource_definition)
        else:
            resource_definition = None

        changed = False
        ml_compute_info = self.get(self.name)

        if self.state == 'present':
            if ml_compute_info:
                # Update
                params_override = self.update_params(kwargs,
                                                     self.UPDATE_OVERRIDES,
                                                     ml_compute_info)

                # Generate ml_compute based on ansible args passed in.
                ml_compute = load_compute(None,
                                          params_override=params_override)
                ml_compute_info = self.entity_to_dict(ml_compute_info)
                changed = not self.default_compare({},
                                                   self.entity_to_dict(ml_compute),
                                                   ml_compute_info,
                                                   '',
                                                   self.results)
                if changed and not self.check_mode:
                    response = self.client.compute.begin_update(ml_compute)
                    ml_compute_object = self.get_poller_result(response)
                    ml_compute_info = self.entity_to_dict(ml_compute_object)

            else:
                # Create
                params_override = self.update_params(kwargs, self.PARAM_OVERRIDES)

                # Generate ml_compute based on ansible args passed in.
                ml_compute = load_compute(resource_definition,
                                          params_override=params_override)
                changed = True
                if not self.check_mode:
                    response = self.client.begin_create_or_update(ml_compute)
                    ml_compute_object = self.get_poller_result(response)
                    ml_compute_info = self.entity_to_dict(ml_compute_object)
        elif self.state == 'absent':
            # Delete
            if ml_compute_info:
                changed = True
                if not self.check_mode:
                    response = self.client.compute.begin_delete(name=self.name)
                    result = self.get_poller_result(response)
                ml_compute_info = self.get(self.name, as_dict=True)
        elif self.state == 'start':
            # Start
            if hasattr(ml_compute_info, 'state'):
                if ml_compute_info.state != "Running":
                    changed = True
                    if not self.check_mode:
                        self.start(self.name)
                ml_compute_info = self.get(self.name, as_dict=True)
            else:
                self.fail("Compute instance doesn't exist")
        elif self.state == 'stop':
            # Stop
            if hasattr(ml_compute_info, 'state'):
                if ml_compute_info.state != "Stopped":
                    changed = True
                    if not self.check_mode:
                        self.stop(self.name)
                ml_compute_info = self.get(self.name, as_dict=True)
            else:
                self.fail("Compute instance doesn't exist")
        elif self.state == 'restart':
            # Restart
            if hasattr(ml_compute_info, 'state'):
                changed = True
                if not self.check_mode:
                    self.restart(self.name)
                ml_compute_info = self.get(self.name, as_dict=True)
            else:
                self.fail("Compute instance doesn't exist")

        self.results['ml_compute'] = ml_compute_info
        self.results['changed'] = changed
        return self.results

    def update_params(self, kwargs, param_overrides, ml_compute=None):
        params_override = []

        # If ml_compute is defined use those values as a base.
        # Update can't change some of these values but we need
        # to keep the same values as before the update.
        if ml_compute:
            items = self.entity_to_dict(ml_compute).items()
            params_override = [{k: v} for k, v in items if k != "ssh_settings"]

        # Update values based on what param_overrides says we can update.
        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if kwargs[key] is not None and key in param_overrides:
                if key == 'vnet_name':
                    params_override.append({"network_settings.vnet_name":
                                            kwargs[key]})
                elif key == 'subnet':
                    params_override.append({"network_settings.subnet":
                                            kwargs[key]})
                elif key == 'admin_username':
                    params_override.append({"ssh_settings.admin_username":
                                            kwargs[key]})
                elif key == 'admin_password':
                    params_override.append({"ssh_settings.admin_password":
                                            kwargs[key]})
                elif key == 'ssh_key_value':
                    params_override.append({"ssh_settings.ssh_key_value":
                                            kwargs[key]})
                elif key == 'user_tenant_id':
                    params_override.append({"create_on_behalf_of.user_tenant_id":
                                            kwargs[key]})
                elif key == 'user_object_id':
                    params_override.append({"create_on_behalf_of.user_object_id":
                                            kwargs[key]})
                elif key == 'identity_type':
                    params_override.append({"identity.type": kwargs[key]})
                elif key == 'user_assigned_identities':
                    identities = self._process_user_assigned_identities(kwargs[key])
                    params_override.append({"identity.user_assigned_identities": identities})
                else:
                    params_override.append({key: kwargs[key]})
        return params_override

    # Convert list to a list of dictionaries
    def _process_user_assigned_identities(self, resource_ids):
        return [{"resource_id": resource_id} for resource_id in resource_ids]

    def get(self, name, as_dict=False):
        try:
            # Attempt to retrieve ml_compute based on name
            ml_compute_info = self.client.compute.get(name)
            if as_dict:
                ml_compute_info = self.entity_to_dict(ml_compute_info)
        except ResourceNotFoundError:
            ml_compute_info = None
        return ml_compute_info

    def start(self, name):
        try:
            response = self.client.compute.begin_start(name=name)
            result = self.get_poller_result(response)
            return result
        except Exception as e:  # pylint: disable=broad-exception-caught
            self.fail("Error start the compute instance: {0}".format(str(e)))

    def stop(self, name):
        try:
            response = self.client.compute.begin_stop(name=name)
            result = self.get_poller_result(response)
            return result
        except Exception as e:  # pylint: disable=broad-exception-caught
            self.fail("Error stop the compute instance: {0}".format(str(e)))

    def restart(self, name):
        try:
            response = self.client.compute.begin_restart(name=name)
            result = self.get_poller_result(response)
            return result
        except Exception as e:  # pylint: disable=broad-exception-caught
            self.fail("Error restart the compute instance: {0}".format(str(e)))


def main():
    AzureRMMLCompute()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The cisco.ios bgp_address_family fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.bgp_address_family.bgp_address_family import (
    Bgp_address_familyArgs,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.rm_templates.bgp_address_family import (
    Bgp_address_familyTemplate,
)


class Bgp_address_familyFacts(object):
    """The cisco.ios_bgp_address_family facts class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = Bgp_address_familyArgs.argument_spec

    def get_bgp_address_family_data(self, connection):
        return connection.get("show running-config | section ^router bgp")

    def _process_facts(self, objs):
        """makes data as per the facts after data obtained from parsers"""
        addr_fam_facts = {}
        processed_af_list = []
        addr_fam_facts["as_number"] = objs.get("as_number")

        if objs.get("address_family"):
            for af_dict in objs["address_family"].values():
                if af_dict.get("afi") in ["ipv4", "ipv6"] and not af_dict.get("safi"):
                    af_dict["safi"] = "unicast"
                if "neighbors" in af_dict:
                    neighbor_list = []
                    for neighbor_address, neighbor_attributes in af_dict["neighbors"].items():
                        if not neighbor_attributes.get("neighbor_address"):
                            neighbor_attributes["neighbor_address"] = neighbor_address
                        neighbor_list.append(neighbor_attributes)
                    af_dict["neighbors"] = neighbor_list
                processed_af_list.append(af_dict)
        if processed_af_list:
            addr_fam_facts["address_family"] = processed_af_list
        return addr_fam_facts

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for Bgp_address_family network resource

        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf

        :rtype: dictionary
        :returns: facts
        """
        facts = {}
        objs = {}

        if not data:
            data = self.get_bgp_address_family_data(connection)

        # parse native config using the Bgp_address_family template
        bgp_af_parser = Bgp_address_familyTemplate(lines=data.splitlines(), module=self._module)
        objs = bgp_af_parser.parse()
        if objs:
            objs = self._process_facts(utils.remove_empties(objs))
        ansible_facts["ansible_network_resources"].pop("bgp_address_family", None)
        params = utils.remove_empties(
            bgp_af_parser.validate_config(self.argument_spec, {"config": objs}, redact=True),
        )
        facts["bgp_address_family"] = params.get("config", {})
        ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts



#!/usr/bin/python

# Copyright (c) 2021, Christian Wollinger <cwollinger@web.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: discord
short_description: Send Discord messages
version_added: 3.1.0
description:
  - Sends a message to a Discord channel using the Discord webhook API.
author: Christian Wollinger (@cwollinger)
seealso:
  - name: API documentation
    description: Documentation for Discord API.
    link: https://discord.com/developers/docs/resources/webhook#execute-webhook
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  webhook_id:
    description:
      - The webhook ID.
      - 'Format from Discord webhook URL: C(/webhooks/{webhook.id}/{webhook.token}).'
    required: true
    type: str
  webhook_token:
    description:
      - The webhook token.
      - 'Format from Discord webhook URL: C(/webhooks/{webhook.id}/{webhook.token}).'
    required: true
    type: str
  content:
    description:
      - Content of the message to the Discord channel.
      - At least one of O(content) and O(embeds) must be specified.
    type: str
  username:
    description:
      - Overrides the default username of the webhook.
    type: str
  avatar_url:
    description:
      - Overrides the default avatar of the webhook.
    type: str
  tts:
    description:
      - Set this to V(true) if this is a TTS (Text to Speech) message.
    type: bool
    default: false
  embeds:
    description:
      - Send messages as Embeds to the Discord channel.
      - Embeds can have a colored border, embedded images, text fields and more.
      - 'Allowed parameters are described in the Discord Docs: U(https://discord.com/developers/docs/resources/channel#embed-object).'
      - At least one of O(content) and O(embeds) must be specified.
    type: list
    elements: dict
"""

EXAMPLES = r"""
- name: Send a message to the Discord channel
  community.general.discord:
    webhook_id: "00000"
    webhook_token: "XXXYYY"
    content: "This is a message from ansible"

- name: Send a message to the Discord channel with specific username and avatar
  community.general.discord:
    webhook_id: "00000"
    webhook_token: "XXXYYY"
    content: "This is a message from ansible"
    username: Ansible
    avatar_url: "https://docs.ansible.com/favicon/favicon-32x32.png"

- name: Send a embedded message to the Discord channel
  community.general.discord:
    webhook_id: "00000"
    webhook_token: "XXXYYY"
    embeds:
      - title: "Embedded message"
        description: "This is an embedded message"
        footer:
          text: "Author: Ansible"
        image:
          url: "https://docs.ansible.com/favicon/favicon-32x32.png"

- name: Send two embedded messages
  community.general.discord:
    webhook_id: "00000"
    webhook_token: "XXXYYY"
    embeds:
      - title: "First message"
        description: "This is my first embedded message"
        footer:
          text: "Author: Ansible"
        image:
          url: "https://docs.ansible.com/favicon/favicon-32x32.png"
      - title: "Second message"
        description: "This is my first second message"
        footer:
          text: "Author: Ansible"
          icon_url: "https://docs.ansible.com/favicon/favicon-32x32.png"
        fields:
          - name: "Field 1"
            value: "Value of my first field"
          - name: "Field 2"
            value: "Value of my second field"
        timestamp: "{{ ansible_date_time.iso8601 }}"
"""

RETURN = r"""
http_code:
  description:
    - Response Code returned by Discord API.
  returned: always
  type: int
  sample: 204
"""

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import fetch_url


def discord_check_mode(module):
    webhook_id = module.params["webhook_id"]
    webhook_token = module.params["webhook_token"]

    headers = {"content-type": "application/json"}

    url = f"https://discord.com/api/webhooks/{webhook_id}/{webhook_token}"

    response, info = fetch_url(module, url, method="GET", headers=headers)
    return response, info


def discord_text_msg(module):
    webhook_id = module.params["webhook_id"]
    webhook_token = module.params["webhook_token"]
    content = module.params["content"]
    user = module.params["username"]
    avatar_url = module.params["avatar_url"]
    tts = module.params["tts"]
    embeds = module.params["embeds"]

    headers = {"content-type": "application/json"}

    url = f"https://discord.com/api/webhooks/{webhook_id}/{webhook_token}"

    payload = {
        "content": content,
        "username": user,
        "avatar_url": avatar_url,
        "tts": tts,
        "embeds": embeds,
    }

    payload = module.jsonify(payload)

    response, info = fetch_url(module, url, data=payload, headers=headers, method="POST")
    return response, info


def main():
    module = AnsibleModule(
        argument_spec=dict(
            webhook_id=dict(type="str", required=True),
            webhook_token=dict(type="str", required=True, no_log=True),
            content=dict(type="str"),
            username=dict(type="str"),
            avatar_url=dict(type="str"),
            tts=dict(type="bool", default=False),
            embeds=dict(type="list", elements="dict"),
        ),
        required_one_of=[["content", "embeds"]],
        supports_check_mode=True,
    )

    if module.check_mode:
        response, info = discord_check_mode(module)
        if info["status"] != 200:
            try:
                module.fail_json(
                    http_code=info["status"], msg=info["msg"], response=module.from_json(info["body"]), info=info
                )
            except Exception:
                module.fail_json(http_code=info["status"], msg=info["msg"], info=info)
        else:
            module.exit_json(
                msg=info["msg"], changed=False, http_code=info["status"], response=module.from_json(response.read())
            )
    else:
        response, info = discord_text_msg(module)
        if info["status"] != 204:
            try:
                module.fail_json(
                    http_code=info["status"], msg=info["msg"], response=module.from_json(info["body"]), info=info
                )
            except Exception:
                module.fail_json(http_code=info["status"], msg=info["msg"], info=info)
        else:
            module.exit_json(msg=info["msg"], changed=True, http_code=info["status"])


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: accessanalyzer_validate_policy_info
version_added: 5.0.0
short_description: Performs validation of IAM policies
description:
  - Requests the validation of a policy and returns a list of findings.
options:
  policy:
    description:
      - A properly json formatted policy.
    type: json
    aliases: ['policy_document']
    required: true
  locale:
    description:
      - The locale to use for localizing the findings.
      - Supported locales include C(DE), C(EN), C(ES), C(FR), C(IT), C(JA), C(KO), C(PT_BR),
        C(ZH_CN) and C(ZH_TW).
      - For more information about supported locales see the AWS Documentation
        C(https://docs.aws.amazon.com/access-analyzer/latest/APIReference/API_ValidatePolicy.html)
    type: str
    required: false
    default: 'EN'
  policy_type:
    description:
      - The type of policy to validate.
      - C(identity) policies grant permissions to IAM principals, including both managed and inline
        policies for IAM roles, users, and groups.
      - C(resource) policies policies grant permissions on AWS resources, including trust policies
        for IAM roles and bucket policies for S3 buckets.
    type: str
    choices: ['identity', 'resource', 'service_control']
    default: 'identity'
    required: false
  resource_type:
    description:
      - The type of resource to attach to your resource policy.
      - Ignored unless I(policy_type=resource).
      - Supported resource types include C(AWS::S3::Bucket), C(AWS::S3::AccessPoint),
        C(AWS::S3::MultiRegionAccessPoint) and C(AWS::S3ObjectLambda::AccessPoint)
      - For resource types not supported as valid values, IAM Access Analyzer runs policy checks
        that apply to all resource policies.
      - For more information about supported locales see the AWS Documentation
        C(https://docs.aws.amazon.com/access-analyzer/latest/APIReference/API_ValidatePolicy.html)
    type: str
    required: false
  results_filter:
    description:
      - Filter the findings and limit them to specific finding types.
    type: list
    elements: str
    choices: ['error', 'security', 'suggestion', 'warning']
    required: false
author:
  - Mark Chappell (@tremble)
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Validate a policy
- name: Validate a simple IAM policy
  community.aws.accessanalyzer_validate_policy_info:
    policy: "{{ lookup('template', 'managed_policy.json.j2') }}"
"""

RETURN = r"""
findings:
  description: The list of findings in a policy returned by IAM Access Analyzer based on its suite of policy checks.
  returned: success
  type: list
  elements: dict
  contains:
    finding_details:
      description:
        - A localized message describing the finding.
      type: str
      returned: success
      sample: 'Resource ARN does not match the expected ARN format. Update the resource portion of the ARN.'
    finding_type:
      description:
        - The severity of the finding.
      type: str
      returned: success
      sample: 'ERROR'
    issue_code:
      description:
        - An identifier for the type of issue found.
      type: str
      returned: success
      sample: 'INVALID_ARN_RESOURCE'
    learn_more_link:
      description:
        - A link to additional information about the finding type.
      type: str
      returned: success
      sample: 'https://docs.aws.amazon.com/IAM/latest/UserGuide/access-analyzer-reference-policy-checks.html'
    locations:
      description:
        - The location of the item resulting in the recommendations.
      type: list
      returned: success
      elements: dict
      contains:
        path:
          description: A path in a policy, represented as a sequence of path elements.
          type: list
          elements: dict
          returned: success
          sample: [{"value": "Statement"}, {"index": 0}, {"value": "Resource"}, {"index": 0}]
        span:
          description:
            - Where in the policy the finding refers to.
            - Note - when using lookups or passing dictionaries to I(policy) the policy string may be
              converted to a single line of JSON, changing th column, line and offset values.
          type: dict
          contains:
            start:
              description: The start position of the span.
              type: dict
              returned: success
              contains:
                column:
                  description: The column of the position, starting from C(0).
                  type: int
                  returned: success
                line:
                  description: The line of the position, starting from C(1).
                  type: int
                  returned: success
                offset:
                  description: The offset within the policy that corresponds to the position, starting from C(0).
                  type: int
                  returned: success
            end:
              description: The end position of the span.
              type: dict
              returned: success
              contains:
                column:
                  description: The column of the position, starting from C(0).
                  type: int
                  returned: success
                line:
                  description: The line of the position, starting from C(1).
                  type: int
                  returned: success
                offset:
                  description: The offset within the policy that corresponds to the position, starting from C(0).
                  type: int
                  returned: success
"""

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def filter_findings(findings, type_filter):
    if not type_filter:
        return findings

    # Convert type_filter to the findingType strings returned by the API
    filter_map = dict(error="ERROR", security="SECURITY_WARNING", suggestion="SUGGESTION", warning="WARNING")
    allowed_types = [filter_map[t] for t in type_filter]

    filtered_results = [f for f in findings if f.get("findingType", None) in allowed_types]
    return filtered_results


def main():
    # Botocore only supports specific values for locale and resource_type, however the supported
    # values are likely to be expanded, let's avoid hard coding limits which might not hold true in
    # the long term...
    argument_spec = dict(
        policy=dict(required=True, type="json", aliases=["policy_document"]),
        locale=dict(required=False, type="str", default="EN"),
        policy_type=dict(
            required=False, type="str", default="identity", choices=["identity", "resource", "service_control"]
        ),
        resource_type=dict(required=False, type="str"),
        results_filter=dict(
            required=False, type="list", elements="str", choices=["error", "security", "suggestion", "warning"]
        ),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)

    policy_type_map = dict(
        identity="IDENTITY_POLICY", resource="RESOURCE_POLICY", service_control="SERVICE_CONTROL_POLICY"
    )

    policy = module.params.get("policy")
    policy_type = policy_type_map[module.params.get("policy_type")]
    locale = module.params.get("locale").upper()
    resource_type = module.params.get("resource_type")
    results_filter = module.params.get("results_filter")

    try:
        client = module.client("accessanalyzer", retry_decorator=AWSRetry.jittered_backoff())
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS")

    params = dict(locale=locale, policyDocument=policy, policyType=policy_type)
    if policy_type == "RESOURCE_POLICY" and resource_type:
        params["policyType"] = resource_type

    results = client.validate_policy(aws_retry=True, **params)

    findings = filter_findings(results.get("findings", []), results_filter)
    results["findings"] = findings

    results = camel_dict_to_snake_dict(results)

    module.exit_json(changed=False, **results)


if __name__ == "__main__":
    main()



# Copyright (c) 2016, Peter Sagerson <psagers@ignorare.net>
# Copyright (c) 2016, Jiri Tyr <jiri.tyr@gmail.com>
# Copyright (c) 2017-2018 Keller Fuchs (@KellerFuchs) <kellerfuchs@hashbang.sh>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this doc fragment is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations


class ModuleDocFragment:
    # Standard LDAP documentation fragment
    DOCUMENTATION = r"""
notes:
  - The default authentication settings attempts to use a SASL EXTERNAL bind over a UNIX domain socket. This works well with
    the default Ubuntu install for example, which includes a C(cn=peercred,cn=external,cn=auth) ACL rule allowing root to
    modify the server configuration. If you need to use a simple bind to access your server, pass the credentials in O(bind_dn)
    and O(bind_pw).
options:
  bind_dn:
    description:
      - A DN to bind with. Try to use a SASL bind with the EXTERNAL mechanism as default when this parameter is omitted.
      - Use an anonymous bind if the parameter is blank.
    type: str
  bind_pw:
    description:
      - The password to use with O(bind_dn).
    type: str
    default: ''
  ca_path:
    description:
      - Set the path to PEM file with CA certs.
    type: path
    version_added: "6.5.0"
  client_cert:
    type: path
    description:
      - PEM formatted certificate chain file to be used for SSL client authentication.
      - Required if O(client_key) is defined.
    version_added: "7.1.0"
  client_key:
    type: path
    description:
      - PEM formatted file that contains your private key to be used for SSL client authentication.
      - Required if O(client_cert) is defined.
    version_added: "7.1.0"
  dn:
    required: true
    description:
      - The DN of the entry to add or remove.
    type: str
  referrals_chasing:
    choices: [disabled, anonymous]
    default: anonymous
    type: str
    description:
      - Set the referrals chasing behavior.
      - V(anonymous) follow referrals anonymously. This is the default behavior.
      - V(disabled) disable referrals chasing. This sets C(OPT_REFERRALS) to off.
    version_added: 2.0.0
  server_uri:
    description:
      - The O(server_uri) parameter may be a comma- or whitespace-separated list of URIs containing only the schema, the host,
        and the port fields.
      - The default value lets the underlying LDAP client library look for a UNIX domain socket in its default location.
      - Note that when using multiple URIs you cannot determine to which URI your client gets connected.
      - For URIs containing additional fields, particularly when using commas, behavior is undefined.
    type: str
    default: ldapi:///
  start_tls:
    description:
      - Use the START_TLS LDAP extension if set to V(true).
    type: bool
    default: false
  validate_certs:
    description:
      - If set to V(false), SSL certificates are not validated.
      - This should only be used on sites using self-signed certificates.
    type: bool
    default: true
  sasl_class:
    description:
      - The class to use for SASL authentication.
    type: str
    choices: ['external', 'gssapi']
    default: external
    version_added: "2.0.0"
  xorder_discovery:
    description:
      - Set the behavior on how to process Xordered DNs.
      - V(enable) performs a C(ONELEVEL) search below the superior RDN to find the matching DN.
      - V(disable) always uses the DN unmodified (as passed by the O(dn) parameter).
      - V(auto) only performs a search if the first RDN does not contain an index number (C({x})).
    type: str
    choices: ['enable', 'auto', 'disable']
    default: auto
    version_added: "6.4.0"
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_cellular_gateway_esims_swap
short_description: Resource module for organizations _cellular _gateway _esims _swap
description:
  - Manage operations create and update of the resource organizations _cellular _gateway
    _esims _swap.
  - Swap which profile an eSIM uses.
  - Get the status of a profile swap.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  id:
    description: Id path parameter. ESIM EID.
    type: str
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
  swaps:
    description: Each object represents a swap for one eSIM.
    elements: dict
    suboptions:
      eid:
        description: ESIM EID.
        type: str
      target:
        description: Target Profile attributes.
        suboptions:
          accountId:
            description: ID of the target account; can be the account currently tied
              to the eSIM.
            type: str
          communicationPlan:
            description: Name of the target communication plan.
            type: str
          ratePlan:
            description: Name of the target rate plan.
            type: str
        type: dict
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for cellularGateway createOrganizationCellularGatewayEsimsSwap
    description: Complete reference of the createOrganizationCellularGatewayEsimsSwap
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-organization-cellular-gateway-esims-swap
  - name: Cisco Meraki documentation for cellularGateway updateOrganizationCellularGatewayEsimsSwap
    description: Complete reference of the updateOrganizationCellularGatewayEsimsSwap
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-organization-cellular-gateway-esims-swap
notes:
  - SDK Method used are
    cellular_gateway.CellularGateway.create_organization_cellular_gateway_esims_swap,
    cellular_gateway.CellularGateway.update_organization_cellular_gateway_esims_swap,
  - Paths used are
    post /organizations/{organizationId}/cellularGateway/esims/swap,
    put /organizations/{organizationId}/cellularGateway/esims/swap/{id},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.organizations_cellular_gateway_esims_swap:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    organizationId: string
    swaps:
      - eid: '1234567890'
        target:
          accountId: '456'
          communicationPlan: A comm plan
          ratePlan: A rate plan
- name: Update by id
  cisco.meraki.organizations_cellular_gateway_esims_swap:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    id: string
    organizationId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "eid": "string",
      "iccid": "string",
      "status": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: lambda_execute
version_added: 5.0.0
short_description: Execute an AWS Lambda function
description:
  - This module executes AWS Lambda functions, allowing synchronous and asynchronous
    invocation.
  - Prior to release 5.0.0 this module was called M(community.aws.execute_lambda).
    The usage did not change.
  - This module was originally added to C(community.aws) in release 1.0.0.
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
author:
  - "Ryan Scott Brown (@ryansb) <ryansb@redhat.com>"
notes:
  - Async invocation will always return an empty RV(result.output)  key.
  - Synchronous invocation may result in a function timeout, resulting in an
    empty RV(result.output) key.
options:
  name:
    description:
      - The name of the function to be invoked. This can only be used for
        invocations within the calling account. To invoke a function in another
        account, use O(function_arn) to specify the full ARN.
    type: str
  function_arn:
    description:
      - The name of the function to be invoked
    type: str
  tail_log:
    description:
      - If O(tail_log=true), the result of the task will include the last 4 KB
        of the CloudWatch log for the function execution. Log tailing only
        works if you use synchronous invocation O(wait=true). This is usually
        used for development or testing Lambdas.
    type: bool
    default: false
  wait:
    description:
      - Whether to wait for the function results or not. If O(wait=no)
        the task will not return any results. To wait for the Lambda function
        to complete, set O(wait=true) and the result will be available in the
        I(output) key.
    type: bool
    default: true
  dry_run:
    description:
      - Do not *actually* invoke the function. A C(DryRun) call will check that
        the caller has permissions to call the function, especially for
        checking cross-account permissions.
    type: bool
    default: false
  version_qualifier:
    description:
      - Which version/alias of the function to run. This defaults to the
        V(LATEST) revision, but can be set to any existing version or alias.
        See U(https://docs.aws.amazon.com/lambda/latest/dg/versioning-aliases.html)
        for details.
    type: str
  payload:
    description:
      - A dictionary in any form to be provided as input to the Lambda function.
    default: {}
    type: dict
"""

EXAMPLES = r"""
- amazon.aws.lambda_execute:
    name: test-function
    # the payload is automatically serialized and sent to the function
    payload:
      foo: bar
      value: 8
  register: response

# Test that you have sufficient permissions to execute a Lambda function in
# another account
- amazon.aws.lambda_execute:
    function_arn: arn:aws:lambda:us-east-1:123456789012:function/some-function
    dry_run: true

- amazon.aws.lambda_execute:
    name: test-function
    payload:
      foo: bar
      value: 8
    wait: true
    tail_log: true
  register: response
  # the response will have a `logs` key that will contain a log (up to 4KB) of the function execution in Lambda

# Pass the Lambda event payload as a json file.
- amazon.aws.lambda_execute:
    name: test-function
    payload: "{{ lookup('file','lambda_event.json') }}"
  register: response

- amazon.aws.lambda_execute:
    name: test-function
    version_qualifier: PRODUCTION
"""

RETURN = r"""
result:
    description: Resulting data structure from a successful task execution.
    returned: success
    type: dict
    contains:
        output:
            description: Function output if O(wait=true) and the function returns a value.
            returned: success
            type: dict
            sample: "{ 'output': 'something' }"
        logs:
            description: The last 4KB of the function logs. Only provided if O(tail_log=true).
            type: str
            returned: if O(tail_log=true)
        status:
            description: V(StatusCode) of API call exit (200 for synchronous invokes, 202 for async).
            type: int
            sample: 200
            returned: always
"""

import base64
import json

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry


def main():
    argument_spec = dict(
        name=dict(),
        function_arn=dict(),
        wait=dict(default=True, type="bool"),
        tail_log=dict(default=False, type="bool"),
        dry_run=dict(default=False, type="bool"),
        version_qualifier=dict(),
        payload=dict(default={}, type="dict"),
    )
    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=[
            ["name", "function_arn"],
        ],
        required_one_of=[("name", "function_arn")],
    )

    name = module.params.get("name")
    function_arn = module.params.get("function_arn")
    await_return = module.params.get("wait")
    dry_run = module.params.get("dry_run")
    tail_log = module.params.get("tail_log")
    version_qualifier = module.params.get("version_qualifier")
    payload = module.params.get("payload")

    try:
        client = module.client("lambda", retry_decorator=AWSRetry.jittered_backoff())
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS")

    invoke_params = {}

    if await_return:
        # await response
        invoke_params["InvocationType"] = "RequestResponse"
    else:
        # fire and forget
        invoke_params["InvocationType"] = "Event"
    if dry_run or module.check_mode:
        # dry_run overrides invocation type
        invoke_params["InvocationType"] = "DryRun"

    if tail_log and await_return:
        invoke_params["LogType"] = "Tail"
    elif tail_log and not await_return:
        module.fail_json(
            msg=(
                "The `tail_log` parameter is only available if "
                "the invocation waits for the function to complete. "
                "Set `wait` to true or turn off `tail_log`."
            )
        )
    else:
        invoke_params["LogType"] = "None"

    if version_qualifier:
        invoke_params["Qualifier"] = version_qualifier

    if payload:
        invoke_params["Payload"] = json.dumps(payload)

    if function_arn:
        invoke_params["FunctionName"] = function_arn
    elif name:
        invoke_params["FunctionName"] = name

    if module.check_mode:
        module.exit_json(changed=True)

    try:
        wait_for_lambda(client, module, name or function_arn)
        response = client.invoke(**invoke_params, aws_retry=True)
    except is_boto3_error_code("ResourceNotFoundException") as nfe:
        module.fail_json_aws(
            nfe,
            msg=(
                "Could not find Lambda to execute. Make sure "
                "the ARN is correct and your profile has "
                "permissions to execute this function."
            ),
        )
    except botocore.exceptions.ClientError as ce:  # pylint: disable=duplicate-except
        module.fail_json_aws(ce, msg="Client-side error when invoking Lambda, check inputs and specific error")
    except botocore.exceptions.ParamValidationError as ve:  # pylint: disable=duplicate-except
        module.fail_json_aws(ve, msg="Parameters to `invoke` failed to validate")
    except botocore.exceptions.BotoCoreError as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg="Unexpected failure while invoking Lambda function")

    results = {
        "logs": "",
        "status": response["StatusCode"],
        "output": "",
    }

    if response.get("LogResult"):
        try:
            # logs are base64 encoded in the API response
            results["logs"] = base64.b64decode(response.get("LogResult", ""))
        except Exception as e:
            module.fail_json_aws(e, msg="Failed while decoding logs")

    if invoke_params["InvocationType"] == "RequestResponse":
        try:
            results["output"] = json.loads(response["Payload"].read().decode("utf8"))
        except Exception as e:
            module.fail_json_aws(e, msg="Failed while decoding function return value")

        if isinstance(results.get("output"), dict) and any(
            [results["output"].get("stackTrace"), results["output"].get("errorMessage")]
        ):
            # AWS sends back stack traces and error messages when a function failed
            # in a RequestResponse (synchronous) context.
            template = (
                "Function executed, but there was an error in the Lambda function. "
                "Message: {errmsg}, Type: {type}, Stack Trace: {trace}"
            )

            error_data = {
                # format the stacktrace sent back as an array into a multiline string
                "trace": "\n".join(results.get("output", {}).get("stackTrace", [])),
                "errmsg": results["output"].get("errorMessage"),
                "type": results["output"].get("errorType"),
            }
            module.fail_json(msg=template.format(**error_data), result=results)

    module.exit_json(changed=True, result=results)


def wait_for_lambda(client, module, name_or_arn):
    try:
        client_active_waiter = client.get_waiter("function_active")
        client_updated_waiter = client.get_waiter("function_updated")
        client_active_waiter.wait(FunctionName=name_or_arn)
        client_updated_waiter.wait(FunctionName=name_or_arn)
    except botocore.exceptions.WaiterError as e:
        module.fail_json_aws(e, msg="Timeout while waiting on lambda to be Active")
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed while waiting on lambda to be Active")


if __name__ == "__main__":
    main()



# Copyright (c) 2016, Yanis Guenane <yanis+ansible@guenane.org>
# Copyright (c) 2020, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import base64
import traceback
import typing as t

from ansible.module_utils.common.text.converters import to_bytes

from ansible_collections.community.crypto.plugins.module_utils._argspec import (
    ArgumentSpec,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.basic import (
    OpenSSLObjectError,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.module_backends.privatekey_info import (
    PrivateKeyConsistencyError,
    PrivateKeyParseError,
    get_privatekey_info,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.pem import (
    identify_private_key_format,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.support import (
    get_fingerprint_of_privatekey,
)
from ansible_collections.community.crypto.plugins.module_utils._cryptography_dep import (
    COLLECTION_MINIMUM_CRYPTOGRAPHY_VERSION,
    assert_required_cryptography_version,
)

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule  # pragma: no cover
    from cryptography.hazmat.primitives.asymmetric.types import (  # pragma: no cover
        PrivateKeyTypes,
    )

    from ansible_collections.community.crypto.plugins.plugin_utils._action_module import (  # pragma: no cover
        AnsibleActionModule,
    )

    GeneralAnsibleModule = t.Union[  # noqa: UP007
        AnsibleModule, AnsibleActionModule
    ]  # pragma: no cover


MINIMAL_CRYPTOGRAPHY_VERSION = COLLECTION_MINIMUM_CRYPTOGRAPHY_VERSION

try:
    import cryptography
    import cryptography.exceptions
    import cryptography.hazmat.backends
    import cryptography.hazmat.primitives.asymmetric.dsa
    import cryptography.hazmat.primitives.asymmetric.ec
    import cryptography.hazmat.primitives.asymmetric.ed448
    import cryptography.hazmat.primitives.asymmetric.ed25519
    import cryptography.hazmat.primitives.asymmetric.rsa
    import cryptography.hazmat.primitives.asymmetric.utils
    import cryptography.hazmat.primitives.asymmetric.x448
    import cryptography.hazmat.primitives.asymmetric.x25519
    import cryptography.hazmat.primitives.serialization
except ImportError:
    pass


class PrivateKeyError(OpenSSLObjectError):
    pass


# From the object called `module`, only the following properties are used:
#
#  - module.params[]
#  - module.warn(msg: str)
#  - module.fail_json(msg: str, **kwargs)


class _Curve:
    def __init__(
        self,
        *,
        name: str,
        ectype: str,
        deprecated: bool,
    ) -> None:
        self.name = name
        self.ectype = ectype
        self.deprecated = deprecated

    def _get_ec_class(
        self, *, module: GeneralAnsibleModule
    ) -> type[cryptography.hazmat.primitives.asymmetric.ec.EllipticCurve]:
        ecclass: (
            type[cryptography.hazmat.primitives.asymmetric.ec.EllipticCurve] | None
        ) = getattr(cryptography.hazmat.primitives.asymmetric.ec, self.ectype, None)
        if ecclass is None:
            module.fail_json(
                msg=f"Your cryptography version does not support {self.ectype}"
            )
        return ecclass

    def create(
        self, *, size: int, module: GeneralAnsibleModule
    ) -> cryptography.hazmat.primitives.asymmetric.ec.EllipticCurve:
        ecclass = self._get_ec_class(module=module)
        return ecclass()

    def verify(
        self,
        *,
        privatekey: cryptography.hazmat.primitives.asymmetric.ec.EllipticCurvePrivateKey,
        module: GeneralAnsibleModule,
    ) -> bool:
        ecclass = self._get_ec_class(module=module)
        return isinstance(privatekey.private_numbers().public_numbers.curve, ecclass)


class PrivateKeyBackend:
    def _add_curve(
        self,
        name: str,
        ectype: str,
        *,
        deprecated: bool = False,
    ) -> None:
        self.curves[name] = _Curve(name=name, ectype=ectype, deprecated=deprecated)

    def __init__(self, *, module: GeneralAnsibleModule) -> None:
        self.module = module
        self.type: t.Literal[
            "DSA", "ECC", "Ed25519", "Ed448", "RSA", "X25519", "X448"
        ] = module.params["type"]
        self.size: int = module.params["size"]
        self.curve: str | None = module.params["curve"]
        self.passphrase: str | None = module.params["passphrase"]
        self.cipher: str = module.params["cipher"]
        self.format: t.Literal["pkcs1", "pkcs8", "raw", "auto", "auto_ignore"] = (
            module.params["format"]
        )
        self.format_mismatch: t.Literal["regenerate", "convert"] = module.params.get(
            "format_mismatch", "regenerate"
        )
        self.regenerate: t.Literal[
            "never", "fail", "partial_idempotence", "full_idempotence", "always"
        ] = module.params.get("regenerate", "full_idempotence")

        self.private_key: PrivateKeyTypes | None = None

        self.existing_private_key: PrivateKeyTypes | None = None
        self.existing_private_key_bytes: bytes | None = None

        self.diff_before = self._get_info(data=None)
        self.diff_after = self._get_info(data=None)

        self.curves: dict[str, _Curve] = {}
        self._add_curve("secp224r1", "SECP224R1")
        self._add_curve("secp256k1", "SECP256K1")
        self._add_curve("secp256r1", "SECP256R1")
        self._add_curve("secp384r1", "SECP384R1")
        self._add_curve("secp521r1", "SECP521R1")
        self._add_curve("secp192r1", "SECP192R1", deprecated=True)
        self._add_curve("sect163k1", "SECT163K1", deprecated=True)
        self._add_curve("sect163r2", "SECT163R2", deprecated=True)
        self._add_curve("sect233k1", "SECT233K1", deprecated=True)
        self._add_curve("sect233r1", "SECT233R1", deprecated=True)
        self._add_curve("sect283k1", "SECT283K1", deprecated=True)
        self._add_curve("sect283r1", "SECT283R1", deprecated=True)
        self._add_curve("sect409k1", "SECT409K1", deprecated=True)
        self._add_curve("sect409r1", "SECT409R1", deprecated=True)
        self._add_curve("sect571k1", "SECT571K1", deprecated=True)
        self._add_curve("sect571r1", "SECT571R1", deprecated=True)
        self._add_curve("brainpoolP256r1", "BrainpoolP256R1", deprecated=True)
        self._add_curve("brainpoolP384r1", "BrainpoolP384R1", deprecated=True)
        self._add_curve("brainpoolP512r1", "BrainpoolP512R1", deprecated=True)

    def _get_info(self, *, data: bytes | None) -> dict[str, t.Any]:
        if data is None:
            return {}
        result: dict[str, t.Any] = {"can_parse_key": False}
        try:
            result.update(
                get_privatekey_info(
                    module=self.module,
                    content=data,
                    passphrase=self.passphrase,
                    return_private_key_data=False,
                    prefer_one_fingerprint=True,
                )
            )
        except PrivateKeyConsistencyError as exc:
            result.update(exc.result)
        except PrivateKeyParseError as exc:
            result.update(exc.result)
        except Exception:
            pass
        return result

    def _get_wanted_format(self) -> t.Literal["pkcs1", "pkcs8", "raw"]:
        if self.format not in ("auto", "auto_ignore"):
            return self.format  # type: ignore
        if self.type in ("X25519", "X448", "Ed25519", "Ed448"):
            return "pkcs8"
        return "pkcs1"

    def generate_private_key(self) -> None:
        """(Re-)Generate private key."""
        try:
            if self.type == "RSA":
                self.private_key = (
                    cryptography.hazmat.primitives.asymmetric.rsa.generate_private_key(
                        public_exponent=65537,  # OpenSSL always uses this
                        key_size=self.size,
                    )
                )
            if self.type == "DSA":
                self.private_key = (
                    cryptography.hazmat.primitives.asymmetric.dsa.generate_private_key(
                        key_size=self.size
                    )
                )
            if self.type == "X25519":
                self.private_key = (
                    cryptography.hazmat.primitives.asymmetric.x25519.X25519PrivateKey.generate()
                )
            if self.type == "X448":
                self.private_key = (
                    cryptography.hazmat.primitives.asymmetric.x448.X448PrivateKey.generate()
                )
            if self.type == "Ed25519":
                self.private_key = (
                    cryptography.hazmat.primitives.asymmetric.ed25519.Ed25519PrivateKey.generate()
                )
            if self.type == "Ed448":
                self.private_key = (
                    cryptography.hazmat.primitives.asymmetric.ed448.Ed448PrivateKey.generate()
                )
            if self.type == "ECC" and self.curve in self.curves:
                if self.curves[self.curve].deprecated:
                    self.module.warn(
                        f"Elliptic curves of type {self.curve} should not be used for new keys!"
                    )
                self.private_key = (
                    cryptography.hazmat.primitives.asymmetric.ec.generate_private_key(
                        curve=self.curves[self.curve].create(
                            size=self.size, module=self.module
                        ),
                    )
                )
        except cryptography.exceptions.UnsupportedAlgorithm:
            self.module.fail_json(
                msg=f"Cryptography backend does not support the algorithm required for {self.type}"
            )

    def convert_private_key(self) -> None:
        """Convert existing private key (self.existing_private_key) to new private key (self.private_key).

        This is effectively a copy without active conversion. The conversion is done
        during load and store; get_private_key_data() uses the destination format to
        serialize the key.
        """
        self._ensure_existing_private_key_loaded()
        self.private_key = self.existing_private_key

    def get_private_key_data(self) -> bytes:
        """Return bytes for self.private_key"""
        if self.private_key is None:
            raise AssertionError("private_key not set")  # pragma: no cover
        # Select export format and encoding
        try:
            export_format_txt = self._get_wanted_format()
            export_encoding = cryptography.hazmat.primitives.serialization.Encoding.PEM
            if export_format_txt == "pkcs1":
                # "TraditionalOpenSSL" format is PKCS1
                export_format = (
                    cryptography.hazmat.primitives.serialization.PrivateFormat.TraditionalOpenSSL
                )
            elif export_format_txt == "pkcs8":
                export_format = (
                    cryptography.hazmat.primitives.serialization.PrivateFormat.PKCS8
                )
            elif export_format_txt == "raw":
                export_format = (
                    cryptography.hazmat.primitives.serialization.PrivateFormat.Raw
                )
                export_encoding = (
                    cryptography.hazmat.primitives.serialization.Encoding.Raw
                )
            else:
                # pylint does not notice that all possible values for export_format_txt have been covered.
                raise AssertionError("Can never be reached")  # pragma: no cover
        except AttributeError:
            self.module.fail_json(
                msg=f'Cryptography backend does not support the selected output format "{self.format}"'
            )

        # Select key encryption
        encryption_algorithm: (
            cryptography.hazmat.primitives.serialization.KeySerializationEncryption
        ) = cryptography.hazmat.primitives.serialization.NoEncryption()
        if self.cipher and self.passphrase:
            if self.cipher == "auto":
                encryption_algorithm = cryptography.hazmat.primitives.serialization.BestAvailableEncryption(
                    to_bytes(self.passphrase)
                )
            else:
                self.module.fail_json(
                    msg='Cryptography backend can only use "auto" for cipher option.'
                )

        # Serialize key
        try:
            return self.private_key.private_bytes(
                encoding=export_encoding,
                format=export_format,
                encryption_algorithm=encryption_algorithm,
            )
        except ValueError:
            self.module.fail_json(
                msg=f'Cryptography backend cannot serialize the private key in the required format "{self.format}"'
            )
        except Exception:
            self.module.fail_json(
                msg=f'Error while serializing the private key in the required format "{self.format}"',
                exception=traceback.format_exc(),
            )

    def set_existing(self, *, privatekey_bytes: bytes | None) -> None:
        """Set existing private key bytes. None indicates that the key does not exist."""
        self.existing_private_key_bytes = privatekey_bytes
        self.diff_after = self.diff_before = self._get_info(
            data=self.existing_private_key_bytes
        )

    def has_existing(self) -> bool:
        """Query whether an existing private key is/has been there."""
        return self.existing_private_key_bytes is not None

    def _load_privatekey(self) -> PrivateKeyTypes:
        data = self.existing_private_key_bytes
        if data is None:
            raise AssertionError(
                "existing_private_key_bytes not set"
            )  # pragma: no cover
        try:
            # Interpret bytes depending on format.
            key_format = identify_private_key_format(data)
            if key_format == "raw":
                if len(data) == 56:
                    return cryptography.hazmat.primitives.asymmetric.x448.X448PrivateKey.from_private_bytes(
                        data
                    )
                if len(data) == 57:
                    return cryptography.hazmat.primitives.asymmetric.ed448.Ed448PrivateKey.from_private_bytes(
                        data
                    )
                if len(data) == 32:
                    if self.type == "X25519":
                        return cryptography.hazmat.primitives.asymmetric.x25519.X25519PrivateKey.from_private_bytes(
                            data
                        )
                    if self.type == "Ed25519":
                        return cryptography.hazmat.primitives.asymmetric.ed25519.Ed25519PrivateKey.from_private_bytes(
                            data
                        )
                    try:
                        return cryptography.hazmat.primitives.asymmetric.x25519.X25519PrivateKey.from_private_bytes(
                            data
                        )
                    except Exception:
                        return cryptography.hazmat.primitives.asymmetric.ed25519.Ed25519PrivateKey.from_private_bytes(
                            data
                        )
                raise PrivateKeyError("Cannot load raw key")

            return cryptography.hazmat.primitives.serialization.load_pem_private_key(
                data,
                None if self.passphrase is None else to_bytes(self.passphrase),
            )
        except Exception as e:
            raise PrivateKeyError(e) from e

    def _ensure_existing_private_key_loaded(self) -> None:
        """Make sure that self.existing_private_key is populated from self.existing_private_key_bytes."""
        if self.existing_private_key is None and self.has_existing():
            self.existing_private_key = self._load_privatekey()

    def _check_passphrase(self) -> bool:
        """Check whether provided passphrase matches, assuming self.existing_private_key_bytes has been populated."""
        if self.existing_private_key_bytes is None:
            raise AssertionError(
                "existing_private_key_bytes not set"
            )  # pragma: no cover
        try:
            key_format = identify_private_key_format(self.existing_private_key_bytes)
            if key_format == "raw":
                # Raw keys cannot be encrypted. To avoid incompatibilities, we try to
                # actually load the key (and return False when this fails).
                self._load_privatekey()
                # Loading the key succeeded. Only return True when no passphrase was
                # provided.
                return self.passphrase is None
            return bool(
                cryptography.hazmat.primitives.serialization.load_pem_private_key(
                    self.existing_private_key_bytes,
                    None if self.passphrase is None else to_bytes(self.passphrase),
                )
            )
        except Exception:
            return False

    def _check_size_and_type(self) -> bool:
        """Check whether provided size and type matches, assuming self.existing_private_key has been populated."""
        if isinstance(
            self.existing_private_key,
            cryptography.hazmat.primitives.asymmetric.rsa.RSAPrivateKey,
        ):
            return (
                self.type == "RSA" and self.size == self.existing_private_key.key_size
            )
        if isinstance(
            self.existing_private_key,
            cryptography.hazmat.primitives.asymmetric.dsa.DSAPrivateKey,
        ):
            return (
                self.type == "DSA" and self.size == self.existing_private_key.key_size
            )
        if isinstance(
            self.existing_private_key,
            cryptography.hazmat.primitives.asymmetric.x25519.X25519PrivateKey,
        ):
            return self.type == "X25519"
        if isinstance(
            self.existing_private_key,
            cryptography.hazmat.primitives.asymmetric.x448.X448PrivateKey,
        ):
            return self.type == "X448"
        if isinstance(
            self.existing_private_key,
            cryptography.hazmat.primitives.asymmetric.ed25519.Ed25519PrivateKey,
        ):
            return self.type == "Ed25519"
        if isinstance(
            self.existing_private_key,
            cryptography.hazmat.primitives.asymmetric.ed448.Ed448PrivateKey,
        ):
            return self.type == "Ed448"
        if isinstance(
            self.existing_private_key,
            cryptography.hazmat.primitives.asymmetric.ec.EllipticCurvePrivateKey,
        ):
            if self.type != "ECC":
                return False
            if self.curve not in self.curves:
                return False
            return self.curves[self.curve].verify(
                privatekey=self.existing_private_key, module=self.module
            )

        return False

    def _check_format(self) -> bool:
        """Check whether the key file format, assuming self.existing_private_key and self.existing_private_key_bytes has been populated."""
        if self.existing_private_key_bytes is None:
            raise AssertionError(
                "existing_private_key_bytes not set"
            )  # pragma: no cover
        if self.format == "auto_ignore":
            return True
        try:
            key_format = identify_private_key_format(self.existing_private_key_bytes)
            return key_format == self._get_wanted_format()
        except Exception:
            return False

    def needs_regeneration(self) -> bool:
        """Check whether a regeneration is necessary."""
        if self.regenerate == "always":
            return True
        if not self.has_existing():
            # key does not exist
            return True
        if not self._check_passphrase():
            if self.regenerate == "full_idempotence":
                return True
            self.module.fail_json(
                msg="Unable to read the key. The key is protected with a another passphrase / no passphrase or broken."
                " Will not proceed. To force regeneration, call the module with `generate`"
                " set to `full_idempotence` or `always`, or with `force=true`."
            )
        self._ensure_existing_private_key_loaded()
        if self.regenerate != "never" and not self._check_size_and_type():
            if self.regenerate in ("partial_idempotence", "full_idempotence"):
                return True
            self.module.fail_json(
                msg="Key has wrong type and/or size."
                " Will not proceed. To force regeneration, call the module with `generate`"
                " set to `partial_idempotence`, `full_idempotence` or `always`, or with `force=true`."
            )
        # During generation step, regenerate if format does not match and format_mismatch == 'regenerate'
        if (
            self.format_mismatch == "regenerate"
            and self.regenerate != "never"
            and not self._check_format()
        ):
            if self.regenerate in ("partial_idempotence", "full_idempotence"):
                return True
            self.module.fail_json(
                msg="Key has wrong format."
                " Will not proceed. To force regeneration, call the module with `generate`"
                " set to `partial_idempotence`, `full_idempotence` or `always`, or with `force=true`."
                " To convert the key, set `format_mismatch` to `convert`."
            )
        return False

    def needs_conversion(self) -> bool:
        """Check whether a conversion is necessary. Must only be called if needs_regeneration() returned False."""
        # During conversion step, convert if format does not match and format_mismatch == 'convert'
        self._ensure_existing_private_key_loaded()
        return (
            self.has_existing()
            and self.format_mismatch == "convert"
            and not self._check_format()
        )

    def _get_fingerprint(self) -> dict[str, str] | None:
        if self.private_key:
            return get_fingerprint_of_privatekey(self.private_key)
        try:
            self._ensure_existing_private_key_loaded()
        except Exception:
            # Ignore errors
            pass
        if self.existing_private_key:
            return get_fingerprint_of_privatekey(self.existing_private_key)
        return None

    def dump(self, *, include_key: bool) -> dict[str, t.Any]:
        """Serialize the object into a dictionary."""

        if not self.private_key:
            try:
                self._ensure_existing_private_key_loaded()
            except Exception:
                # Ignore errors
                pass
        result: dict[str, t.Any] = {
            "type": self.type,
            "size": self.size,
            "fingerprint": self._get_fingerprint(),
        }
        if self.type == "ECC":
            result["curve"] = self.curve
        # Get hold of private key bytes
        pk_bytes = self.existing_private_key_bytes
        if self.private_key is not None:
            pk_bytes = self.get_private_key_data()
        self.diff_after = self._get_info(data=pk_bytes)
        if include_key:
            # Store result
            if pk_bytes:
                if identify_private_key_format(pk_bytes) == "raw":
                    result["privatekey"] = base64.b64encode(pk_bytes)
                else:
                    result["privatekey"] = pk_bytes.decode("utf-8")
            else:
                result["privatekey"] = None

        result["diff"] = {
            "before": self.diff_before,
            "after": self.diff_after,
        }
        return result


def select_backend(module: GeneralAnsibleModule) -> PrivateKeyBackend:
    assert_required_cryptography_version(
        module, minimum_cryptography_version=MINIMAL_CRYPTOGRAPHY_VERSION
    )
    return PrivateKeyBackend(module=module)


def get_privatekey_argument_spec() -> ArgumentSpec:
    return ArgumentSpec(
        argument_spec={
            "size": {"type": "int", "default": 4096},
            "type": {
                "type": "str",
                "default": "RSA",
                "choices": ["DSA", "ECC", "Ed25519", "Ed448", "RSA", "X25519", "X448"],
            },
            "curve": {
                "type": "str",
                "choices": [
                    "secp224r1",
                    "secp256k1",
                    "secp256r1",
                    "secp384r1",
                    "secp521r1",
                    "secp192r1",
                    "brainpoolP256r1",
                    "brainpoolP384r1",
                    "brainpoolP512r1",
                    "sect163k1",
                    "sect163r2",
                    "sect233k1",
                    "sect233r1",
                    "sect283k1",
                    "sect283r1",
                    "sect409k1",
                    "sect409r1",
                    "sect571k1",
                    "sect571r1",
                ],
            },
            "passphrase": {"type": "str", "no_log": True},
            "cipher": {"type": "str", "default": "auto"},
            "format": {
                "type": "str",
                "default": "auto_ignore",
                "choices": ["pkcs1", "pkcs8", "raw", "auto", "auto_ignore"],
            },
            "format_mismatch": {
                "type": "str",
                "default": "regenerate",
                "choices": ["regenerate", "convert"],
            },
            "select_crypto_backend": {
                "type": "str",
                "choices": ["auto", "cryptography"],
                "default": "auto",
            },
            "regenerate": {
                "type": "str",
                "default": "full_idempotence",
                "choices": [
                    "never",
                    "fail",
                    "partial_idempotence",
                    "full_idempotence",
                    "always",
                ],
            },
        },
        required_if=[
            ("type", "ECC", ["curve"]),
        ],
    )


__all__ = (
    "PrivateKeyBackend",
    "PrivateKeyError",
    "get_privatekey_argument_spec",
    "select_backend",
)



#!/usr/bin/python
#
# Copyright (c) 2020 Guopeng Lin, <linguopeng1998@gmail.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_adapplication

version_added: "1.6.0"

short_description: Manage Azure Active Directory application

description:
    - Manage Azure Active Directory application.

options:
    app_id:
        description:
            - Application ID.
        type: str

    display_name:
        description:
            - The display name of the application.
        required: true
        type: str

    app_roles:
        description:
            - Declare the roles you want to associate with your application.
        type: list
        elements: dict
        suboptions:
            allowed_member_types:
                description:
                    - Specifies whether this app role can be assigned to users and groups I(allowed_member_types=User).
                    - To other application's I(allowed_member_types=Application).
                    - Or both C(User) and C(Appplication).
                type: list
                elements: str
                required: True
            description:
                description:
                    - The description for the app role.
                    - This is displayed when the app role is being assigned.
                    - if the app role functions as an application permission, during consent experiences.
                type: str
            display_name:
                description:
                    - Display name for the permission that appears in the app role assignment and consent experiences.
                type: str
            is_enabled:
                description:
                    - When creating or updating an app role, this must be set to true (which is the default).
                    - To delete a role, this must first be set to false.
                    - At that point, in a subsequent call, this role may be removed.
                type: bool
            value:
                description:
                    - Specifies the value to include in the roles claim in ID tokens and access tokens authenticating an assigned user or service principal.
                    - Must not exceed 120 characters in length.
                    - Allowed characters include ! # $ % & ' ( ) * + , - . / : ; < = > ? @ [ ] ^ + _ ` { | } ~, and characters in the ranges 0-9, A-Z and a-z.
                    - Any other character, including the space character, are not allowed.
                type: str

    sign_in_audience:
        description:
            - The application can be used from any Azure AD tenants.
            - Microsoft Graph SDK deprecate I(available_to_other_tenants), replace by I(sign_in_audience).
            - Refer to link U(https://learn.microsoft.com/en-us/graph/migrate-azure-ad-graph-property-differences#application-property-differences)
        type: str
        choices:
            - AzureADMyOrg
            - AzureADMultipleOrgs
            - AzureADandPersonalMicrosoftAccount
            - PersonalMicrosoftAccount

    credential_description:
        description:
            - The description of the password.
        type: str

    end_date:
        description:
            - Date or datetime after which credentials expire(e.g. '2017-12-31').
            - Default value is one year after current time.
        type: str

    homepage:
        description:
            - The url where users can sign in and use your app.
        type: str

    identifier_uris:
        description:
            - Space-separated unique URIs that Azure AD can use for this app.
        elements: str
        type: list

    key_type:
        description:
            - The type of the key credentials associated with the application.
        type: str
        default: AsymmetricX509Cert
        choices:
            - AsymmetricX509Cert
            - Password
            - Symmetric

    key_usage:
        description:
            - The usage of the key credentials associated with the application.
        type: str
        default: Verify
        choices:
            - Sign
            - Verify

    key_value:
        description:
            - The value for the key credentials associated with the application.
        type: str

    native_app:
        description:
            - An application which can be installed on a user's device or computer.
        type: bool

    notes:
        description:
            - Notes relevant for the management of the application.
        type: str

    oauth2_allow_implicit_flow:
        description:
            - Whether to allow implicit grant flow for OAuth2.
        type: bool

    optional_claims:
        description:
            - Declare the optional claims for the application.
        type: dict
        suboptions:
            access_token_claims :
                description:
                    - The optional claims returned in the JWT access token
                type: list
                elements: dict
                suboptions:
                    name:
                        description:
                            - The name of the optional claim.
                        type: str
                        required: True
                    source:
                        description:
                            - The source (directory object) of the claim.
                            - There are predefined claims and user-defined claims from extension properties.
                            - If the source value is null, the claim is a predefined optional claim.
                            - If the source value is user, the value in the name property is the extension property from the user object.
                        type: str
                    essential:
                        description:
                            - If the value is true, the claim specified by the client is necessary to ensure a smooth authorization experience\
                               for the specific task requested by the end user.
                            - The default value is false.
                        default: false
                        type: bool
                    additional_properties:
                        description:
                            - Additional properties of the claim.
                            - If a property exists in this collection, it modifies the behavior of the optional claim specified in the name property.
                        type: list
                        elements: str
            id_token_claims:
                description:
                    - The optional claims returned in the JWT ID token
                type: list
                elements: dict
                suboptions:
                    name:
                        description:
                            - The name of the optional claim.
                        type: str
                        required: True
                    source:
                        description:
                            - The source (directory object) of the claim.
                            - There are predefined claims and user-defined claims from extension properties.
                            - If the source value is null, the claim is a predefined optional claim.
                            - If the source value is user, the value in the name property is the extension property from the user object.
                        type: str
                    essential:
                        description:
                            - If the value is true, the claim specified by the client is necessary to ensure a smooth authorization experience\
                               for the specific task requested by the end user.
                            - The default value is false.
                        default: false
                        type: bool
                    additional_properties:
                        description:
                            - Additional properties of the claim.
                            - If a property exists in this collection, it modifies the behavior of the optional claim specified in the name property.
                        type: list
                        elements: str
            saml2_token_claims:
                description:
                    - The optional claims returned in the SAML token
                type: list
                elements: dict
                suboptions:
                    name:
                        description:
                            - The name of the optional claim.
                        type: str
                        required: True
                    source:
                        description:
                            - The source (directory object) of the claim.
                            - There are predefined claims and user-defined claims from extension properties.
                            - If the source value is null, the claim is a predefined optional claim.
                            - If the source value is user, the value in the name property is the extension property rom the user object.
                        type: str
                    essential:
                        description:
                            - If the value is true, the claim specified by the client is necessary to ensure a smooth authorization experience\
                               for the specific task requested by the end user.
                            - The default value is false.
                        default: false
                        type: bool
                    additional_properties:
                        description:
                            - Additional properties of the claim.
                            - If a property exists in this collection, it modifies the behavior of the optional claim specified in the name property.
                        type: list
                        elements: str
    password:
        description:
            - (Deprecated). Microsoft Graph generates client secrets server-side and rejects client-supplied values.
            - When set, the supplied value is ignored and a server-generated secret is created instead.
            - Prefer C(request_password=true) with C(password_display_name). The generated secret appears in
              the registered result under C(generated_password_credentials[0].secret_text).
            - For ongoing rotation/management of secrets, use I(azure.azcollection.azure_rm_adpassword).
        type: str
    password_display_name:
        description:
            - Name for the client secret created on the new application.
            - Used together with C(request_password=true) (or legacy C(password)).
        type: str
    key_display_name:
        description:
            - Name for the certificate credential created on the new application.
            - Used together with C(key_value).
        type: str
    request_password:
        description:
            - When C(true), ask Microsoft Graph to generate a client secret for the new application.
            - The generated secret is returned in C(generated_password_credentials[0].secret_text) and is
              only retrievable from that response; there is no way to fetch it later.
            - Mutually exclusive with C(key_value).
            - Has no effect when C(state=absent) or C(native_app=true).
            - On an update, this parameter is ignored with a warning. Microsoft Graph does not allow adding password
              credentials via PATCH /applications. For ongoing secret rotation, use I(azure.azcollection.azure_rm_adpassword).
        type: bool
        default: false

    web_reply_urls:
        description:
            - The web redirect urls.
            - Space-separated URIs to which Azure AD will redirect in response to an OAuth 2.0 request.
            - The value does not need to be a physical endpoint, but must be a valid URI.
        type: list
        elements: str
        aliases:
            - reply_urls

    spa_reply_urls:
        description:
            - The spa redirect urls.
            - Space-separated URIs to which Azure AD will redirect in response to an OAuth 2.0 request.
            - The value does not need to be a physical endpoint, but must be a valid URI.
        type: list
        elements: str

    public_client_reply_urls:
        description:
            - The public client redirect urls.
            - Space-separated URIs to which Azure AD will redirect in response to an OAuth 2.0 request.
            - The value does not need to be a physical endpoint, but must be a valid URI.
        type: list
        elements: str

    required_resource_accesses:
        description:
            - Resource scopes and roles the application requires access to.
            - Should be in manifest json format.
        type: list
        elements: dict
        suboptions:
            resource_app_id:
                description:
                    - The unique identifier for the resource that the application requires access to.
                    - This should be equal to the appId declared on the target resource application.
                type: str
            resource_access:
                description:
                    - The description of the app role.
                type: list
                elements: dict
                suboptions:
                    id:
                        description:
                            - The unique identifier for one of the oauth2PermissionScopes or appRole instances that the resource application exposes.
                        type: str
                    type:
                        description:
                            - Specifies whether the id property references an oauth2PermissionScopes or an appRole.
                            - Possible values are Scope or Role.
                        type: str

    start_date:
        description:
            - Date or datetime at which credentials become valid, such as '2017-01-01'.
            - Default value is current time.
        type: str
    allow_guests_sign_in:
        description:
            - A property on the application to indicate if the application accepts other IDPs or not or partially accepts.
        type: bool
    state:
        description:
            - Assert the state of Active Dirctory service principal.
            - Use C(present) to create or update a Password and use C(absent) to delete.
        default: present
        choices:
            - absent
            - present
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    guopeng_lin (@guopenglin)
    haiyuan_zhang (@haiyuazhang)
    Fred-sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Create ad application
  azure_rm_adapplication:
    display_name: "{{ display_name }}"

- name: Create ad application with multi redirect urls
  azure_rm_adapplication:
    display_name: "{{ display_name }}"
    web_reply_urls:
      - https://web01.com
    spa_reply_urls:
      - https://spa01.com
      - https://spa02.com
    public_client_reply_urls:
      - https://public01.com
      - https://public02.com

- name: Create application with more parameter
  azure_rm_adapplication:
    display_name: "{{ display_name }}"
    sign_in_audience: AzureADandPersonalMicrosoftAccount
    credential_description: "for test"
    end_date: 2021-10-01
    start_date: 2021-05-18
    identifier_uris: fredtest02.com

- name: delete ad application
  azure_rm_adapplication:
    app_id: "{{ app_id }}"
    state: absent

- name: Add certificate credential when creating ad application
  azure_rm_adapplication:
  display_name: "{{ display_name }}"
  key_type: AsymmetricX509Cert
  key_usage: Verify
  key_value: "{{ cert_file.content }}" # slurp gives base64 of the file; module normalizes to DER bytes
  tenant: "{{ tenant_id }}"
  subscription_id: "{{ subscription_id }}"

- name: Create ad application and request a server-generated client secret
  azure_rm_adapplication:
    display_name: "{{ display_name }}"
    request_password: true
    password_display_name: "ansible-managed-secret"
  register: app
  no_log: true

- name: Show the generated secret (only available on this create response)
  ansible.builtin.debug:
    msg: "{{ app.generated_password_credentials[0].secret_text }}"
  no_log: true
'''

RETURN = '''
display_name:
    description:
        - Object's display name or its prefix.
    type: str
    returned: always
    sample: fredAKSCluster
app_id:
    description:
        - The application ID.
    returned: always
    type: str
    sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
object_id:
    description:
        - Object ID of the application
    returned: always
    type: str
    sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
sign_in_audience:
    description:
        - The application can be used from any Azure AD tenants.
    returned: always
    type: str
    sample: AzureADandPersonalMicrosoftAccount
homepage:
    description:
        - The url where users can sign in and use your app.
    returned: always
    type: str
    sample: null
identifier_uris:
    description:
        - Space-separated unique URIs that Azure AD can use for this app.
    returned: always
    type: list
    sample: []
oauth2_allow_implicit_flow:
    description:
        - Whether to allow implicit grant flow for OAuth2.
    returned: always
    type: bool
    sample: false
public_client_reply_urls:
    description:
        - The public client redirect urls.
        - Space-separated URIs to which Azure AD will redirect in response to an OAuth 2.0 request.
    returned: always
    type: list
    sample: []
web_reply_urls:
    description:
        - The web redirect urls.
        - Space-separated URIs to which Azure AD will redirect in response to an OAuth 2.0 request.
    returned: always
    type: list
    sample: []
spa_reply_urls:
    description:
        - The spa redirect urls.
        - Space-separated URIs to which Azure AD will redirect in response to an OAuth 2.0 request.
    returned: always
    type: list
    sample: []
optional_claims:
    description:
        - Declare the optional claims for the application.
    type: complex
    returned: always
    contains:
        access_token_claims :
            description:
                - The optional claims returned in the JWT access token
            type: list
            returned: always
            sample: ['name': 'aud', 'source': null, 'essential': false, 'additional_properties': []]
        id_token_claims:
            description:
                - The optional claims returned in the JWT ID token
            type: list
            returned: always
            sample: ['name': 'acct', 'source': null, 'essential': false, 'additional_properties': []]
        saml2_token_claims:
            description:
                - The optional claims returned in the SAML token
            type: list
            returned: always
            sample: ['name': 'acct', 'source': null, 'essential': false, 'additional_properties': []]
password_credentials:
    description:
        - Client secret credentials on the application (without secret values).
        - For newly-generated secrets on create, see C(generated_password_credentials).
    type: list
    elements: dict
    returned: always
    contains:
        key_id:
            description: Unique identifier (GUID) of the secret.
            type: str
        display_name:
            description: Name of the secret.
            type: str
        hint:
            description: First three characters of the secret value.
            type: str
        start_date_time:
            description: When the secret becomes valid.
            type: str
        end_date_time:
            description: When the secret expires.
            type: str
key_credentials:
    description:
        - Certificate credentials on the application (without raw key bytes).
    type: list
    elements: dict
    returned: always
    contains:
        key_id:
            description: Unique identifier (GUID) of the credential.
            type: str
        display_name:
            description: Name of the credential.
            type: str
        type:
            description: Credential type (e.g. C(AsymmetricX509Cert)).
            type: str
        usage:
            description: Credential usage (e.g. C(Verify)).
            type: str
        start_date_time:
            description: When the credential becomes valid.
            type: str
        end_date_time:
            description: When the credential expires.
            type: str
generated_password_credentials:
    description:
        - Newly-generated client secrets returned on create. Each entry includes C(secret_text), the only
          opportunity to retrieve the generated password value.
        - Treat C(secret_text) as sensitive and register the task with C(no_log) when consuming it.
        - Empty/absent when the module did not request a new secret or when called on update.
    type: list
    elements: dict
    returned: on create when a password credential was requested
    contains:
        key_id:
            description: Unique identifier (GUID) of the new secret.
            type: str
        display_name:
            description: Name of the new secret.
            type: str
        hint:
            description: First three characters of the secret value.
            type: str
        secret_text:
            description: The generated secret value. Only present on initial create.
            type: str
        start_date_time:
            description: When the secret becomes valid.
            type: str
        end_date_time:
            description: When the secret expires.
            type: str
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt

try:
    import datetime
    import base64
    import re
    import dateutil.parser
    import uuid
    import asyncio
    from dateutil.relativedelta import relativedelta
    from msgraph.generated.applications.applications_request_builder import ApplicationsRequestBuilder
    from msgraph.generated.models.application import Application
    from msgraph.generated.models.password_credential import PasswordCredential
    from msgraph.generated.models.key_credential import KeyCredential
    from msgraph.generated.models.required_resource_access import RequiredResourceAccess
    from msgraph.generated.models.resource_access import ResourceAccess
    from msgraph.generated.models.app_role import AppRole
    from msgraph.generated.models.web_application import WebApplication
    from msgraph.generated.models.spa_application import SpaApplication
    from msgraph.generated.models.public_client_application import PublicClientApplication
    from msgraph.generated.models.implicit_grant_settings import ImplicitGrantSettings
    from msgraph.generated.models.optional_claim import OptionalClaim
    from msgraph.generated.models.optional_claims import OptionalClaims
except ImportError:
    # This is handled in azure_rm_common
    pass

app_role_spec = dict(
    allowed_member_types=dict(
        type='list',
        elements='str',
        required=True
    ),
    description=dict(
        type='str'
    ),
    display_name=dict(
        type='str'
    ),
    is_enabled=dict(
        type='bool'
    ),
    value=dict(
        type='str'
    )
)

claims_spec = dict(
    name=dict(
        type='str',
        required=True
    ),
    source=dict(
        type='str'
    ),
    essential=dict(
        type='bool',
        default=False
    ),
    additional_properties=dict(
        type='list',
        elements='str'
    )
)

required_resource_accesses_spec = dict(
    resource_app_id=dict(
        type='str'
    ),
    resource_access=dict(
        type='list',
        elements='dict',
        options=dict(
            id=dict(
                type='str'
            ),
            type=dict(
                type='str'
            )
        )
    )
)


class AzureRMADApplication(AzureRMModuleBaseExt):
    def __init__(self):

        self.module_arg_spec = dict(
            app_id=dict(type='str'),
            display_name=dict(type='str', required=True),
            app_roles=dict(type='list', elements='dict', options=app_role_spec),
            sign_in_audience=dict(
                type='str',
                choices=['AzureADMyOrg', 'AzureADMultipleOrgs', 'AzureADandPersonalMicrosoftAccount', 'PersonalMicrosoftAccount']
            ),
            credential_description=dict(type='str'),
            end_date=dict(type='str'),
            homepage=dict(type='str'),
            allow_guests_sign_in=dict(type='bool'),
            identifier_uris=dict(type='list', elements='str'),
            key_type=dict(type='str', default='AsymmetricX509Cert',
                          choices=['AsymmetricX509Cert', 'Password', 'Symmetric']),
            key_usage=dict(type='str', default='Verify', choices=['Sign', 'Verify']),
            key_value=dict(type='str', no_log=True),
            native_app=dict(type='bool'),
            notes=dict(type='str'),
            oauth2_allow_implicit_flow=dict(type='bool'),
            optional_claims=dict(
                type='dict',
                options=dict(
                    access_token_claims=dict(type='list', elements='dict', no_log=True, options=claims_spec),
                    id_token_claims=dict(type='list', elements='dict', no_log=True, options=claims_spec),
                    saml2_token_claims=dict(type='list', elements='dict', no_log=True, options=claims_spec),
                )
            ),
            password=dict(type='str', no_log=True),
            password_display_name=dict(type='str', no_log=False),
            key_display_name=dict(type='str', no_log=False),
            request_password=dict(type='bool', default=False, no_log=False),
            public_client_reply_urls=dict(type='list', elements='str'),
            web_reply_urls=dict(type='list', elements='str', aliases=['reply_urls']),
            spa_reply_urls=dict(type='list', elements='str'),
            start_date=dict(type='str'),
            required_resource_accesses=dict(type='list', elements='dict', options=required_resource_accesses_spec),
            state=dict(type='str', default='present', choices=['present', 'absent']),
        )

        self.state = None
        self.app_id = None
        self.display_name = None
        self.app_roles = None
        self.credential_description = None
        self.end_date = None
        self.homepage = None
        self.identifier_uris = None
        self.key_type = None
        self.key_usage = None
        self.key_value = None
        self.native_app = None
        self.notes = None
        self.oauth2_allow_implicit_flow = None
        self.optional_claims = None
        self.password = None
        self.password_display_name = None
        self.key_display_name = None
        self.request_password = None
        self.public_client_reply_urls = None
        self.spa_reply_urls = None
        self.web_reply_urls = None
        self.start_date = None
        self.required_resource_accesses = None
        self.allow_guests_sign_in = None
        self.results = dict(changed=False)
        self._client = None
        self.sign_in_audience = None

        super(AzureRMADApplication, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                   supports_check_mode=False,
                                                   supports_tags=False,
                                                   is_ad_resource=True)

    def exec_module(self, **kwargs):
        self._client = self.get_msgraph_client()

        # Guard against the AZURE_COMMON_ARGS 'password' (user-password auth) leaking in via module_defaults.
        if kwargs.get('password') and kwargs.get('ad_user'):
            self.module.warn(
                "Ignoring 'password' on azure_rm_adapplication because 'ad_user' is also set; the value "
                "appears to come from Azure user-password authentication rather than an explicit app secret. "
            )
            kwargs['password'] = None

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        # Microsoft Graph generates secrets server-side; the legacy 'password' value is never honored.
        if self.password and self.state == 'present' and not self.native_app:
            self.module.deprecate(
                "Setting 'password' as a client secret value on azure_rm_adapplication is incompatible "
                "with Microsoft Graph (the value is ignored server-side). Use 'request_password=true' "
                "with 'password_display_name' and read the generated secret from "
                "'generated_password_credentials[0].secret_text', or manage secrets with "
                "azure_rm_adpassword. This parameter will be removed in a future major release.",
                version='4.0.0',
                collection_name='azure.azcollection',
            )
            self.request_password = True

        response = self.get_resource()
        if response:
            if self.state == 'present':
                if self.check_update(response):
                    self.update_resource(response)
            elif self.state == 'absent':
                self.delete_resource(response)
        else:
            if self.state == 'present':
                self.create_resource()
            elif self.state == 'absent':
                self.log("try to delete non exist resource")

        return self.results

    def create_resource(self):
        try:
            key_creds, password_creds, required_accesses, app_roles, optional_claims = None, None, None, None, None
            if self.native_app:
                if self.identifier_uris:
                    self.fail("'identifier_uris' is not required for creating a native application")
            else:
                password_creds, key_creds = self.build_application_creds(
                    key_value=self.key_value,
                    key_type=self.key_type,
                    key_usage=self.key_usage,
                    start_date=self.start_date,
                    end_date=self.end_date,
                    key_description=self.credential_description,
                    request_password=bool(self.request_password),
                    password_display_name=self.password_display_name,
                    key_display_name=self.key_display_name,
                )
            if self.required_resource_accesses:
                required_accesses = self.build_application_accesses(self.required_resource_accesses)

            if self.app_roles:
                app_roles = self.build_app_roles(self.app_roles)

            if self.optional_claims:
                optional_claims = self.build_optional_claims(self.optional_claims)

            create_app = Application(
                sign_in_audience=self.sign_in_audience,
                web=WebApplication(
                    home_page_url=self.homepage,
                    redirect_uris=self.web_reply_urls,
                    implicit_grant_settings=ImplicitGrantSettings(
                        enable_access_token_issuance=self.oauth2_allow_implicit_flow,
                    ),
                ),
                spa=SpaApplication(redirect_uris=self.spa_reply_urls),
                public_client=PublicClientApplication(redirect_uris=self.public_client_reply_urls),
                display_name=self.display_name,
                identifier_uris=self.identifier_uris,
                key_credentials=key_creds,
                notes=self.notes,
                password_credentials=password_creds,
                required_resource_access=required_accesses,
                app_roles=app_roles,
                optional_claims=optional_claims
                # allow_guests_sign_in=self.allow_guests_sign_in,
            )
            response = asyncio.get_event_loop().run_until_complete(self.create_application(create_app))
            self.results['changed'] = True
            self.results.update(self.to_dict(response))
            # Capture the server-generated secret values; only available on the create response.
            self.results['generated_password_credentials'] = self._extract_generated_passwords(response)
            return response
        except Exception as ge:
            self.fail("Error creating application, display_name: {0} - {1}".format(self.display_name, str(ge)))

    def update_resource(self, old_response):
        try:
            key_creds, required_accesses, app_roles, optional_claims = None, None, None, None
            if not self.native_app:
                if self.key_value:
                    key_bytes = self._normalize_cert_key_value(self.key_value)
                    custom_key_id = (self.encode_custom_key_description(self.credential_description)
                                     if self.credential_description else None)
                    key_creds = [KeyCredential(
                        display_name=self.key_display_name,
                        key=key_bytes,
                        usage=(self.key_usage or 'Verify'),
                        type=(self.key_type or 'AsymmetricX509Cert'),
                        custom_key_identifier=custom_key_id,
                    )]
                if self.request_password or self.password:
                    self.module.warn(
                        "Ignoring 'request_password'/'password' on update: Microsoft Graph does not "
                        "allow adding password credentials via PATCH /applications. Use "
                        "azure.azcollection.azure_rm_adpassword to add or rotate client secrets on an "
                        "existing application."
                    )
            if self.required_resource_accesses:
                required_accesses = self.build_application_accesses(self.required_resource_accesses)

            if self.app_roles:
                app_roles = self.build_app_roles(self.app_roles)

            if self.optional_claims:
                optional_claims = self.build_optional_claims(self.optional_claims)

            app_update_param = Application(
                sign_in_audience=self.sign_in_audience,
                web=WebApplication(
                    home_page_url=self.homepage,
                    redirect_uris=self.web_reply_urls,
                    implicit_grant_settings=ImplicitGrantSettings(
                        enable_access_token_issuance=self.oauth2_allow_implicit_flow,
                    ),
                ),
                spa=SpaApplication(redirect_uris=self.spa_reply_urls),
                public_client=PublicClientApplication(redirect_uris=self.public_client_reply_urls),
                display_name=self.display_name,
                identifier_uris=self.identifier_uris,
                key_credentials=key_creds,
                notes=self.notes,
                required_resource_access=required_accesses,
                # allow_guests_sign_in=self.allow_guests_sign_in,
                app_roles=app_roles,
                optional_claims=optional_claims)
            asyncio.get_event_loop().run_until_complete(self.update_application(
                obj_id=old_response['object_id'], update_app=app_update_param))

            self.results['changed'] = True
            self.results.update(self.get_resource())

        except Exception as ge:
            self.fail("Error updating the application app_id {0} - {1}".format(self.app_id, str(ge)))

    def delete_resource(self, response):
        try:
            asyncio.get_event_loop().run_until_complete(self.delete_application(response.get('object_id')))
            self.results['changed'] = True
            return True
        except Exception as ge:
            self.fail(
                "Error deleting application app_id {0} display_name {1} - {2}".format(self.app_id, self.display_name,
                                                                                      str(ge)))

    def get_resource(self):
        try:
            existing_apps = []
            if self.app_id:
                ret = asyncio.get_event_loop().run_until_complete(self.get_application_by_app_id(self.app_id))
                existing_apps = ret.value

            if not existing_apps:
                return False
            result = existing_apps[0]
            return self.to_dict(result)
        except Exception as ge:
            self.fail(ge)
            # self.log("Did not find the graph instance instance {0} - {1}".format(self.app_id, str(ge)))
            # return False

    def check_update(self, response):
        for key in list(self.module_arg_spec.keys()):
            attr = getattr(self, key)
            if attr and key in response:
                if (response and attr != response[key]) or response[key] is None:
                    return True
        return False

    def serialize_claims(self, claims):
        if claims is None:
            return None
        return [{
            "additional_properties": claim.additional_properties,
            "essential": claim.essential,
            "name": claim.name,
            "source": claim.source} for claim in claims]

    def to_dict(self, object):
        app_roles = [{
            'id': app_role.id,
            'display_name': app_role.display_name,
            'is_enabled': app_role.is_enabled,
            'value': app_role.value,
            "description": app_role.description
        } for app_role in object.app_roles]
        optional_claims = {
            "access_token": self.serialize_claims(object.optional_claims.access_token),
            "id_token": self.serialize_claims(object.optional_claims.id_token),
            "saml2_token": self.serialize_claims(object.optional_claims.saml2_token)
        } if object.optional_claims is not None else object.optional_claims
        return dict(
            app_id=object.app_id,
            object_id=object.id,
            display_name=object.display_name,
            app_roles=app_roles,
            sign_in_audience=object.sign_in_audience,
            homepage=object.web.home_page_url,
            identifier_uris=object.identifier_uris,
            notes=object.notes,
            oauth2_allow_implicit_flow=object.web.implicit_grant_settings.enable_access_token_issuance,
            optional_claims=optional_claims,
            # allow_guests_sign_in=object.allow_guests_sign_in,
            web_reply_urls=object.web.redirect_uris,
            spa_reply_urls=object.spa.redirect_uris,
            public_client_reply_urls=object.public_client.redirect_uris,
            password_credentials=self._summarize_password_credentials(object.password_credentials),
            key_credentials=self._summarize_key_credentials(object.key_credentials),
        )

    @staticmethod
    def _summarize_password_credentials(creds):
        if not creds:
            return []
        return [{
            'key_id': str(c.key_id) if c.key_id is not None else None,
            'display_name': c.display_name,
            'hint': c.hint,
            'start_date_time': str(c.start_date_time) if c.start_date_time is not None else None,
            'end_date_time': str(c.end_date_time) if c.end_date_time is not None else None,
        } for c in creds]

    @staticmethod
    def _summarize_key_credentials(creds):
        if not creds:
            return []
        return [{
            'key_id': str(c.key_id) if c.key_id is not None else None,
            'display_name': c.display_name,
            'type': c.type,
            'usage': c.usage,
            'start_date_time': str(c.start_date_time) if c.start_date_time is not None else None,
            'end_date_time': str(c.end_date_time) if c.end_date_time is not None else None,
        } for c in creds]

    @staticmethod
    def _extract_generated_passwords(application):
        # Pull server-generated secret_text from the create response; only returned once by Graph.
        creds = getattr(application, 'password_credentials', None) or []
        out = []
        for c in creds:
            if not getattr(c, 'secret_text', None):
                continue
            out.append({
                'key_id': str(c.key_id) if c.key_id is not None else None,
                'display_name': c.display_name,
                'hint': c.hint,
                'secret_text': c.secret_text,
                'start_date_time': str(c.start_date_time) if c.start_date_time is not None else None,
                'end_date_time': str(c.end_date_time) if c.end_date_time is not None else None,
            })
        return out

    def build_application_creds(self, key_value=None, key_type=None, key_usage=None,
                                start_date=None, end_date=None, key_description=None,
                                request_password=False, password_display_name=None,
                                key_display_name=None):
        # Build credential payloads for POST /applications. Microsoft Graph generates
        # keyId / secretText / hint server-side, so we never send them on create.
        if request_password and key_value:
            self.fail('specify either a password credential (request_password / password) or key_value, but not both.')

        if not start_date:
            start_date = datetime.datetime.utcnow()
        elif isinstance(start_date, str):
            start_date = dateutil.parser.parse(start_date)

        if not end_date:
            end_date = start_date + relativedelta(years=1) - relativedelta(hours=24)
        elif isinstance(end_date, str):
            end_date = dateutil.parser.parse(end_date)

        key_type = key_type or 'AsymmetricX509Cert'
        key_usage = key_usage or 'Verify'

        password_creds = None
        key_creds = None

        if request_password:
            password_creds = [PasswordCredential(
                display_name=password_display_name,
                start_date_time=start_date,
                end_date_time=end_date,
            )]
        elif key_value:
            key_bytes = self._normalize_cert_key_value(key_value)
            custom_key_id = self.encode_custom_key_description(key_description) if key_description else None
            key_creds = [KeyCredential(
                display_name=key_display_name,
                start_date_time=start_date,
                end_date_time=end_date,
                key=key_bytes,
                usage=key_usage,
                type=key_type,
                custom_key_identifier=custom_key_id,
            )]

        return password_creds, key_creds

    def encode_custom_key_description(self, key_description):
        # utf16 is used by AAD portal. Do not change it to other random encoding
        # unless you know what you are doing.
        return key_description.encode('utf-16')

    def _normalize_cert_key_value(self, key_value):
        """
        Normalize a certificate to raw DER bytes suitable for KeyCredential.key.

        Accepts:
        - raw bytes: already DER, returned as-is
        - str (PEM): strip headers and base64-decode to DER bytes
        - str (base64 DER): base64-decode to DER bytes
        - other types: best-effort bytes conversion

        Returns:
        DER bytes
        """
        if isinstance(key_value, bytes):
            return key_value

        if not isinstance(key_value, str):
            try:
                return bytes(key_value)
            except Exception:
                return str(key_value).encode("utf-8")

        s = key_value.strip()

        # PEM with BEGIN/END headers
        m = re.search(r"-----BEGIN CERTIFICATE-----([\s\S]*?)-----END CERTIFICATE-----", s, re.S)
        if m:
            pem_b64 = re.sub(r"\s+", "", m.group(1))
            return base64.b64decode(pem_b64)

        # Base64 DER
        try:
            return base64.b64decode(s)
        except Exception:
            # Last resort: treat as UTF-8 bytes
            return s.encode("utf-8")

    def gen_guid(self):
        return uuid.uuid4()

    def build_application_accesses(self, required_resource_accesses):
        if not required_resource_accesses:
            return None
        required_accesses = []
        if isinstance(required_resource_accesses, dict):
            self.log('Getting "requiredResourceAccess" from a full manifest')
            required_resource_accesses = required_resource_accesses.get('required_resource_access', [])
        for x in required_resource_accesses:
            accesses = [ResourceAccess(id=y['id'], type=y['type']) for y in x['resource_access']]
            required_accesses.append(RequiredResourceAccess(resource_app_id=x['resource_app_id'],
                                                            resource_access=accesses))
        return required_accesses

    def build_app_roles(self, app_roles):
        if not app_roles:
            return None
        result = []
        if isinstance(app_roles, dict):
            self.log('Getting "appRoles" from a full manifest')
            app_roles = app_roles.get('appRoles', [])
        for x in app_roles:
            role = AppRole(id=x.get('id', None) or self.gen_guid(),
                           allowed_member_types=x.get('allowed_member_types', None),
                           description=x.get('description', None), display_name=x.get('display_name', None),
                           is_enabled=x.get('is_enabled', None), value=x.get('value', None))  # value ? additional_data
            result.append(role)
        return result

    def build_optional_claims(self, optional_claims):

        def build_claims(claims_dict):
            if claims_dict is None:
                return None
            return [OptionalClaim(
                essential=claim.get("essential"),
                name=claim.get("name"),
                source=claim.get("source"),
                additional_properties=claim.get("additional_properties")
            ) for claim in claims_dict]

        claims = OptionalClaims(
            access_token=build_claims(optional_claims.get("access_token_claims")),
            id_token=build_claims(optional_claims.get("id_token_claims")),
            saml2_token=build_claims(optional_claims.get("saml2_token_claims"))
        )
        return claims

    async def create_application(self, creat_app):
        return await self._client.applications.post(body=creat_app)

    async def update_application(self, obj_id, update_app):
        return await self._client.applications.by_application_id(obj_id).patch(body=update_app)

    async def get_application_by_app_id(self, app_id):
        request_configuration = ApplicationsRequestBuilder.ApplicationsRequestBuilderGetRequestConfiguration(
            query_parameters=ApplicationsRequestBuilder.ApplicationsRequestBuilderGetQueryParameters(
                filter=(" appId eq '{0}'".format(app_id)), ),
        )

        return await self._client.applications.get(request_configuration=request_configuration)

    async def delete_application(self, obj_id):
        await self._client.applications.by_application_id(obj_id).delete()

    async def get_applications(self, filters):
        request_configuration = ApplicationsRequestBuilder.ApplicationsRequestBuilderGetRequestConfiguration(
            query_parameters=ApplicationsRequestBuilder.ApplicationsRequestBuilderGetQueryParameters(
                filter=(' and '.join(filters))
            ))
        return await self._client.applications.get(request_configuration=request_configuration)


def main():
    AzureRMADApplication()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_subordinate_ca
short_description: Manages subordinate-ca objects on Checkpoint over Web Services API
description:
  - Manages subordinate-ca objects on Checkpoint devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
  - Available from R82.10 management version.
version_added: "6.7.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  name:
    description:
      - Object name.
    type: str
    required: True
  base64_certificate:
    description:
      - Certificate file encoded in base64.
    type: str
  automatic_enrollment:
    description:
      - Certificate automatic enrollment.
    type: dict
    suboptions:
      automatically_enroll_certificate:
        description:
          - Whether to automatically enroll certificate.
        type: bool
      protocol:
        description:
          - Protocol that communicates with the certificate authority. Available only if "automatically-enroll-certificate" parameter is set to true.
        type: str
        choices: ['scep', 'cmpv2', 'cmpv1']
      scep_settings:
        description:
          - Scep protocol settings. Available only if "protocol" is set to "scep".
        type: dict
        suboptions:
          ca_identifier:
            description:
              - Certificate authority identifier.
            type: str
          url:
            description:
              - Certificate authority URL.
            type: str
      cmpv1_settings:
        description:
          - Cmpv1 protocol settings. Available only if "protocol" is set to "cmpv1".
        type: dict
        suboptions:
          direct_tcp_settings:
            description:
              - Direct tcp transport layer settings.
            type: dict
            suboptions:
              ip_address:
                description:
                  - Certificate authority IP address.
                type: str
              port:
                description:
                  - Port number.
                type: int
      cmpv2_settings:
        description:
          - Cmpv2 protocol settings. Available only if "protocol" is set to "cmpv1".
        type: dict
        suboptions:
          transport_layer:
            description:
              - Transport layer.
            type: str
            choices: ['http', 'direct-tcp']
          direct_tcp_settings:
            description:
              - Direct tcp transport layer settings.
            type: dict
            suboptions:
              ip_address:
                description:
                  - Certificate authority IP address.
                type: str
              port:
                description:
                  - Port number.
                type: int
          http_settings:
            description:
              - Http transport layer settings.
            type: dict
            suboptions:
              url:
                description:
                  - Certificate authority URL.
                type: str
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  domains_to_process:
    description:
      - Indicates which domains to process the commands on. It cannot be used with the details-level full, must be run from the System Domain only and
        with ignore-warnings true. Valid values are, CURRENT_DOMAIN, ALL_DOMAINS_ON_THIS_SERVER.
    type: list
    elements: str
  groups:
    description:
      - Collection of group identifiers.
    type: list
    elements: str
  tags:
    description:
      - Collection of tag identifiers.
    type: list
    elements: str
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-subordinate-ca
  cp_mgmt_subordinate_ca:
    automatic_enrollment:
      automatically_enroll_certificate: true
      cmpv2_settings:
        http_settings:
          url: url
        transport_layer: http
      protocol: cmpv2
    base64_certificate:
      "MIICwjCCAaqgAwIBAgIILdexblpVEMIwDQYJKoZIhvcNAQELBQAwGDEWMBQGA1UEAxMNd3d3Lm9wc2VjLmNvbTAeFw0yMzA2MjUwOTE3MDBaFw0yNTAzMzExNjAwMDBaMBgxFjAUBgNVBAMTDXd3dy5vcH
       lYy5jb20wggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQCjpqCxDaVg+I1b+wqnmjjYtL3v7Tlu/YpMbsKnv+M1gRz6QFUOoSVnxKLo0A7Y4kCqa1OPcHO/LtXuok43F1YZPVKm3xWpY8FmqGqf5
       uGmSwm1HPObcMjwGOyFgwpwEDF5e0UMZ7xtJF8BZ5KKBh3ZfQ1FbmbVqSUPcmOi+NE4JspPlHxX+m6es/yeSGR1A2ezKY7KePTlwVtDe8hiLrYyKG92nka5rkD1QyEIVJ0W5wrnU4nGEDIHeOfT09zroQx
       NLkb51sl4Tog/qw+EraVGIBe/iFnSJoDF37i2mLJqI/t8bel+aGDAxgMx1pO85OClgjPSWL0UIXGI2xrR+JAgMBAAGjEDAOMAwGA1UdEwQFMAMBAf8wDQYJKoZIhvcNAQELBQADggEBAHTs1AutAmSLHF2
       RLJtrRNkso0lMyA7XI7k1TNpTk7TCZLNY0VbUliGbcl+POH4EG8ARUrftnwRDCTBd2BdJTqG2CyNADi+bw8aLvbxok7KH0GlQvGjyfq+sHK12wTl4ULNyYoAPZ01GhXOvkobROdSyjxvBVhxdVo90kj7mH
       v3N83huNhfstDFUBcQCmMkbLuzDUZrl2a1OtqlOdNC6mNvb7Jq9W9vRxGA514e7jqyoM+PwHu5fILx/jmGT8suOUnvbtcDdFhjqixAPer6uSPR0CSbiJvuDy72DPH5mjZK5dQKewNYOZ/BQEsRIBe+Q6eG
       oJqi+cD63cwlw0DCc="
    name: TestSubordinateCa
    state: present

- name: set-subordinate-ca
  cp_mgmt_subordinate_ca:
    automatic_enrollment:
      cmpv1_settings:
        direct_tcp_settings:
          ip_address: 1.1.1.1
      protocol: cmpv1
    name: TestSubordinateCa
    state: present

- name: delete-subordinate-ca
  cp_mgmt_subordinate_ca:
    name: TestSubordinateCa
    state: absent
"""

RETURN = """
cp_mgmt_subordinate_ca:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_objects, api_call


def main():
    argument_spec = dict(
        name=dict(type='str', required=True),
        base64_certificate=dict(type='str'),
        automatic_enrollment=dict(type='dict', options=dict(
            automatically_enroll_certificate=dict(type='bool'),
            protocol=dict(type='str', choices=['scep', 'cmpv2', 'cmpv1']),
            scep_settings=dict(type='dict', options=dict(
                ca_identifier=dict(type='str'),
                url=dict(type='str')
            )),
            cmpv1_settings=dict(type='dict', options=dict(
                direct_tcp_settings=dict(type='dict', options=dict(
                    ip_address=dict(type='str'),
                    port=dict(type='int')
                ))
            )),
            cmpv2_settings=dict(type='dict', options=dict(
                transport_layer=dict(type='str', choices=['http', 'direct-tcp']),
                direct_tcp_settings=dict(type='dict', options=dict(
                    ip_address=dict(type='str'),
                    port=dict(type='int')
                )),
                http_settings=dict(type='dict', options=dict(
                    url=dict(type='str')
                ))
            ))
        )),
        color=dict(type='str', choices=['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green',
                                        'khaki', 'orchid', 'dark orange', 'dark sea green', 'pink', 'turquoise', 'dark blue', 'firebrick', 'brown',
                                        'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon', 'coral', 'sea green',
                                        'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna',
                                        'yellow']),
        comments=dict(type='str'),
        details_level=dict(type='str', choices=['uid', 'standard', 'full']),
        domains_to_process=dict(type='list', elements='str'),
        groups=dict(type='list', elements='str'),
        tags=dict(type='list', elements='str'),
        ignore_warnings=dict(type='bool'),
        ignore_errors=dict(type='bool')
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)
    api_call_object = 'subordinate-ca'

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_ssh_policy
short_description: SSH Policy configuration for Cisco Intersight
description:
  - Manages SSH Policy configuration on Cisco Intersight.
  - A policy to configure SSH service settings on Cisco Intersight managed servers.
  - This policy is applicable only for UCS Servers (Standalone).
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/ssh/Policies/get/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles and Policies that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the SSH Policy.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the SSH Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  enable_ssh:
    description:
      - State of SSH service on the endpoint.
      - If set to false, SSH service will be disabled and ssh_port and ssh_timeout will be ignored.
    type: bool
    default: true
  ssh_port:
    description:
      - Port used for secure shell access.
      - Valid range is 1-65535.
      - This parameter is only used when enable_ssh is true.
    type: int
    default: 22
  ssh_timeout:
    description:
      - Number of seconds to wait before the system considers a SSH request to have timed out.
      - Valid range is 60-10800 seconds.
      - This parameter is only used when enable_ssh is true.
    type: int
    default: 1800
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create SSH Policy with default settings
  cisco.intersight.intersight_ssh_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "SSH-Policy-Default"
    description: "SSH policy with default settings"
    tags:
      - Key: "Environment"
        Value: "Production"
    enable_ssh: true
    state: present

- name: Create SSH Policy with custom port and timeout
  cisco.intersight.intersight_ssh_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "SSH-Policy-Custom"
    description: "SSH policy with custom port and timeout"
    enable_ssh: true
    ssh_port: 2222
    ssh_timeout: 3600
    state: present

- name: Create SSH Policy with SSH disabled
  cisco.intersight.intersight_ssh_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "SSH-Policy-Disabled"
    description: "SSH policy with SSH service disabled"
    enable_ssh: false
    state: present

- name: Delete SSH Policy
  cisco.intersight.intersight_ssh_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "SSH-Policy-Default"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "SSH-Policy-Default",
        "ObjectType": "ssh.Policy",
        "Enabled": true,
        "Port": 22,
        "Timeout": 1800,
        "Tags": [
            {
                "Key": "Environment",
                "Value": "Production"
            }
        ]
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        enable_ssh=dict(type='bool', default=True),
        ssh_port=dict(type='int', default=22),
        ssh_timeout=dict(type='int', default=1800)
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True
    )

    # Validate ssh_port range
    if module.params['ssh_port'] < 1 or module.params['ssh_port'] > 65535:
        module.fail_json(msg="ssh_port must be between 1 and 65535")

    # Validate ssh_timeout range
    if module.params['ssh_timeout'] < 60 or module.params['ssh_timeout'] > 10800:
        module.fail_json(msg="ssh_timeout must be between 60 and 10800 seconds")

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Resource path used to configure policy
    resource_path = '/ssh/Policies'

    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name'],
        'Enabled': intersight.module.params['enable_ssh'],
    }

    if intersight.module.params['state'] == 'present':
        intersight.set_tags_and_description()

        # Add SSH port and timeout only if SSH is enabled
        if intersight.module.params['enable_ssh']:
            intersight.api_body['Port'] = intersight.module.params['ssh_port']
            intersight.api_body['Timeout'] = intersight.module.params['ssh_timeout']

    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: nxos_aaa_server_host
extends_documentation_fragment:
- cisco.nxos.nxos
short_description: Manages AAA server host-specific configuration.
description:
- Manages AAA server host-specific configuration.
version_added: 1.0.0
author: Jason Edelman (@jedelman8)
notes:
- Tested against NXOSv 7.3.(0)D1(1) on VIRL
- Limited Support for Cisco MDS
- Changes to the host key (shared secret) are not idempotent for type 0.
- If C(state=absent) removes the whole host configuration.
options:
  server_type:
    description:
    - The server type is either radius or tacacs.
    required: true
    choices:
    - radius
    - tacacs
    type: str
  address:
    description:
    - Address or name of the radius or tacacs host.
    required: true
    type: str
  key:
    description:
    - Shared secret for the specified host or keyword 'default'.
    type: str
  encrypt_type:
    description:
    - The state of encryption applied to the entered key. O for clear text, 7 for
      encrypted. Type-6 encryption is not supported.
    choices:
    - '0'
    - '7'
    type: str
  host_timeout:
    description:
    - Timeout period for specified host, in seconds or keyword 'default. Range is
      1-60.
    type: str
  auth_port:
    description:
    - Alternate UDP port for RADIUS authentication or keyword 'default'.
    type: str
  acct_port:
    description:
    - Alternate UDP port for RADIUS accounting or keyword 'default'.
    type: str
  tacacs_port:
    description:
    - Alternate TCP port TACACS Server or keyword 'default'.
    type: str
  state:
    description:
    - Manage the state of the resource.
    default: present
    choices:
    - present
    - absent
    type: str
"""
EXAMPLES = """
# Radius Server Host Basic settings
- name: Radius Server Host Basic settings
  cisco.nxos.nxos_aaa_server_host:
    state: present
    server_type: radius
    address: 1.2.3.4
    acct_port: 2084
    host_timeout: 10

# Radius Server Host Key Configuration
- name: Radius Server Host Key Configuration
  cisco.nxos.nxos_aaa_server_host:
    state: present
    server_type: radius
    address: 1.2.3.4
    key: hello
    encrypt_type: 7

# TACACS Server Host Configuration
- name: Tacacs Server Host Configuration
  cisco.nxos.nxos_aaa_server_host:
    state: present
    server_type: tacacs
    tacacs_port: 89
    host_timeout: 10
    address: 5.6.7.8
"""

RETURN = """
proposed:
    description: k/v pairs of parameters passed into module
    returned: always
    type: dict
    sample: {"address": "1.2.3.4", "auth_port": "2084",
            "host_timeout": "10", "server_type": "radius"}
existing:
    description:
        - k/v pairs of existing configuration
    returned: always
    type: dict
    sample: {}
end_state:
    description: k/v pairs of configuration after module execution
    returned: always
    type: dict
    sample: {"address": "1.2.3.4", "auth_port": "2084",
            "host_timeout": "10", "server_type": "radius"}
updates:
    description: command sent to the device
    returned: always
    type: list
    sample: ["radius-server host 1.2.3.4 auth-port 2084 timeout 10"]
changed:
    description: check to see if a change was made on the device
    returned: always
    type: bool
    sample: true
"""
import re

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    get_capabilities,
    load_config,
    run_commands,
)


def execute_show_command(command, module):
    device_info = get_capabilities(module)
    network_api = device_info.get("network_api", "nxapi")

    if network_api == "cliconf":
        cmds = [command]
        body = run_commands(module, cmds)
    elif network_api == "nxapi":
        cmds = {"command": command, "output": "text"}
        body = run_commands(module, cmds)

    return body


def flatten_list(command_lists):
    flat_command_list = []
    for command in command_lists:
        if isinstance(command, list):
            flat_command_list.extend(command)
        else:
            flat_command_list.append(command)
    return flat_command_list


def get_aaa_host_info(module, server_type, address):
    aaa_host_info = {}
    command = "show run | inc {0}-server.host.{1}".format(server_type, address)

    body = execute_show_command(command, module)[0]
    if body:
        try:
            if "radius" in body:
                pattern = (
                    r"\S+ host \S+(?:\s+key 7\s+(\S+))?(?:\s+auth-port (\d+))?"
                    r"(?:\s+acct-port (\d+))?(?:\s+authentication)?"
                    r"(?:\s+accounting)?(?:\s+timeout (\d+))?"
                )
                match = re.search(pattern, body)
                aaa_host_info["key"] = match.group(1)
                if aaa_host_info["key"]:
                    aaa_host_info["key"] = aaa_host_info["key"].replace('"', "")
                    aaa_host_info["encrypt_type"] = "7"
                aaa_host_info["auth_port"] = match.group(2)
                aaa_host_info["acct_port"] = match.group(3)
                aaa_host_info["host_timeout"] = match.group(4)
            elif "tacacs" in body:
                pattern = (
                    r"\S+ host \S+(?:\s+key 7\s+(\S+))?(?:\s+port (\d+))?(?:\s+timeout (\d+))?"
                )
                match = re.search(pattern, body)
                aaa_host_info["key"] = match.group(1)
                if aaa_host_info["key"]:
                    aaa_host_info["key"] = aaa_host_info["key"].replace('"', "")
                    aaa_host_info["encrypt_type"] = "7"
                aaa_host_info["tacacs_port"] = match.group(2)
                aaa_host_info["host_timeout"] = match.group(3)

            aaa_host_info["server_type"] = server_type
            aaa_host_info["address"] = address
        except TypeError:
            return {}
    else:
        return {}

    return aaa_host_info


def config_aaa_host(server_type, address, params, existing):
    cmds = []
    cmd_str = "{0}-server host {1}".format(server_type, address)
    cmd_no_str = "no " + cmd_str

    key = params.get("key")
    enc_type = params.get("encrypt_type", "")

    defval = False
    nondef = False

    if key:
        if key != "default":
            cmds.append(cmd_str + " key {0} {1}".format(enc_type, key))
        else:
            cmds.append(cmd_no_str + " key 7 {0}".format(existing.get("key")))

    locdict = {
        "auth_port": "auth-port",
        "acct_port": "acct-port",
        "tacacs_port": "port",
        "host_timeout": "timeout",
    }

    # platform CLI needs the keywords in the following order
    for key in ["auth_port", "acct_port", "tacacs_port", "host_timeout"]:
        item = params.get(key)
        if item:
            if item != "default":
                cmd_str += " {0} {1}".format(locdict.get(key), item)
                nondef = True
            else:
                cmd_no_str += " {0} 1".format(locdict.get(key))
                defval = True
    if defval:
        cmds.append(cmd_no_str)
    if nondef or not existing:
        cmds.append(cmd_str)

    return cmds


def main():
    argument_spec = dict(
        server_type=dict(choices=["radius", "tacacs"], required=True),
        address=dict(type="str", required=True),
        key=dict(type="str", no_log=False),
        encrypt_type=dict(type="str", choices=["0", "7"]),
        host_timeout=dict(type="str"),
        auth_port=dict(type="str"),
        acct_port=dict(type="str"),
        tacacs_port=dict(type="str"),
        state=dict(choices=["absent", "present"], default="present"),
    )

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    warnings = list()

    server_type = module.params["server_type"]
    address = module.params["address"]
    key = module.params["key"]
    encrypt_type = module.params["encrypt_type"]
    host_timeout = module.params["host_timeout"]
    auth_port = module.params["auth_port"]
    acct_port = module.params["acct_port"]
    tacacs_port = module.params["tacacs_port"]
    state = module.params["state"]

    args = dict(
        server_type=server_type,
        address=address,
        key=key,
        encrypt_type=encrypt_type,
        host_timeout=host_timeout,
        auth_port=auth_port,
        acct_port=acct_port,
        tacacs_port=tacacs_port,
    )

    proposed = dict((k, v) for k, v in args.items() if v is not None)
    changed = False

    if encrypt_type and not key:
        module.fail_json(msg="encrypt_type must be used with key")

    if tacacs_port and server_type != "tacacs":
        module.fail_json(msg="tacacs_port can only be used with server_type=tacacs")

    if (auth_port or acct_port) and server_type != "radius":
        module.fail_json(
            msg="auth_port and acct_port can only be used" "when server_type=radius",
        )

    existing = get_aaa_host_info(module, server_type, address)
    end_state = existing

    commands = []
    delta = {}
    if state == "present":
        if not existing:
            delta = proposed
        else:
            for key, value in proposed.items():
                if key == "encrypt_type":
                    delta[key] = value
                if value != existing.get(key):
                    if value != "default" or existing.get(key):
                        delta[key] = value

        command = config_aaa_host(server_type, address, delta, existing)
        if command:
            commands.append(command)

    elif state == "absent":
        intersect = dict(set(proposed.items()).intersection(existing.items()))
        if intersect.get("address") and intersect.get("server_type"):
            command = "no {0}-server host {1}".format(
                intersect.get("server_type"),
                intersect.get("address"),
            )
            commands.append(command)

    cmds = flatten_list(commands)
    if cmds:
        if module.check_mode:
            module.exit_json(changed=True, commands=cmds)
        else:
            changed = True
            load_config(module, cmds)
            end_state = get_aaa_host_info(module, server_type, address)

    results = {}
    results["proposed"] = proposed
    results["existing"] = existing
    results["updates"] = cmds
    results["changed"] = changed
    results["warnings"] = warnings
    results["end_state"] = end_state

    module.exit_json(**results)


if __name__ == "__main__":
    main()
